@@ -1,5 +1,5 @@
 /*
- * $Id: gfs.h 4747 2010-06-13 06:59:49Z tatebe $
+ * $Id: gfs.h 4703 2010-06-01 11:15:54Z n-soda $
  */
 
 /*
@@ -68,6 +68,11 @@ int gfs_desc_fileno(GFS_Desc);
 #if 0 /* not yet on Gfarm v2 */
 #define GFARM_FILE_APPEND		0x00000800
 #define GFARM_FILE_EXCLUSIVE		0x00001000
+#endif
+#ifdef GFARM_INTERNAL_USE /* internal use only */
+#define GFARM_FILE_TRUNC_PENDING	0x00800000 /* used by gfmd only */
+#endif
+#if 0
 /* the followings are just hints */
 #define GFARM_FILE_SEQUENTIAL		0x01000000
 #define GFARM_FILE_REPLICATE		0x02000000
@@ -271,10 +276,36 @@ gfarm_error_t gfs_getxmlent(struct gfs_xmlattr_ctx *ctxp,
 	char **fpathp, char **namep);
 gfarm_error_t gfs_closexmlattr(struct gfs_xmlattr_ctx *ctxp);
 
-gfarm_error_t gfs_replicate_to(const char *, char *, int);
-gfarm_error_t gfs_replicate_from_to(const char *, char *, int, char *, int);
-gfarm_error_t gfs_migrate_to(const char *, char *, int);
-gfarm_error_t gfs_migrate_from_to(const char *, char *, int, char *, int);
+gfarm_error_t gfs_replicate_to(char *, char *, int);
+gfarm_error_t gfs_replicate_from_to(char *, char *, int, char *, int);
+gfarm_error_t gfs_migrate_to(char *, char *, int);
+gfarm_error_t gfs_migrate_from_to(char *, char *, int, char *, int);
+
+#define GFS_REPLICATE_FILE_FORCE			1	/* no BUSY */
+#ifdef GFARM_INTERNAL_USE /* internal use only */
+#define GFS_REPLICATE_FILE_WAIT				2
+#define GFS_REPLICATE_FILE_MIGRATE			4
+#endif
+gfarm_error_t gfs_replicate_file_from_to_request(
+	const char *, const char *, const char *, int);
+gfarm_error_t gfs_replicate_file_to_request(const char *, const char *, int);
+gfarm_error_t gfs_replicate_file_from_to(
+	const char *, const char *, const char *, int);
+gfarm_error_t gfs_replicate_file_to(const char *, const char *, int);
+
+#define GFS_REPLICA_INFO_INCLUDING_DEAD_HOST		1
+#define GFS_REPLICA_INFO_INCLUDING_INCOMPLETE_COPY	2
+#define GFS_REPLICA_INFO_INCLUDING_DEAD_COPY		4
+struct gfs_replica_info;
+gfarm_error_t gfs_replica_info_by_name(const char *, int,
+	struct gfs_replica_info **);
+int gfs_replica_info_number(struct gfs_replica_info *);
+const char *gfs_replica_info_nth_host(struct gfs_replica_info *, int);
+gfarm_uint64_t gfs_replica_info_nth_gen(struct gfs_replica_info *, int);
+int gfs_replica_info_nth_is_incomplete(struct gfs_replica_info *, int);
+int gfs_replica_info_nth_is_dead_host(struct gfs_replica_info *, int);
+void gfs_replica_info_free(struct gfs_replica_info *);
+
 gfarm_error_t gfs_replica_list_by_name(const char *, int *, char ***);
 gfarm_error_t gfs_replica_remove_by_file(const char *, const char *);
 gfarm_error_t gfs_replicate_to_local(GFS_File, char *, int);
