@@ -1,5 +1,5 @@
 /*
- * $Id: config.c 4769 2010-06-23 19:45:14Z tatebe $
+ * $Id: config.c 4640 2010-04-12 12:34:22Z n-soda $
  */
 
 #include <sys/types.h>
@@ -74,8 +74,8 @@ map_user(gfarm_stringlist map_file_list, char *from, char **to_p,
 	for (i = 0; i < list_len; i++) {
 		mapfile = gfarm_stringlist_elem(&map_file_list, i);
 		if ((map = fopen(mapfile, "r")) == NULL) {
-			gflog_error(GFARM_MSG_1000009, "%s: %s",
-			    mapfile, strerror(errno));
+			gflog_error(GFARM_MSG_1000009,
+			    "%s: cannot open: %s", mapfile, strerror(errno));
 			return (GFARM_ERR_CANT_OPEN);
 		}
 		lineno = 0;
@@ -219,8 +219,12 @@ set_string(char **var, char *value)
 	if (*var != NULL)
 		free(*var);
 	*var = strdup(value);
-	if (*var == NULL)
+	if (*var == NULL) {
+		gflog_debug(GFARM_MSG_1000918,
+			"allocation of memory failed: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
 		return (GFARM_ERR_NO_MEMORY);
+	}
 	return (GFARM_ERR_NO_ERROR);
 }
 
@@ -351,8 +355,6 @@ char *gfarm_localfs_datadir = NULL;
 
 /* miscellaneous */
 #define GFARM_LOG_MESSAGE_VERBOSE_DEFAULT	0
-#define GFARM_NO_FILE_SYSTEM_NODE_TIMEOUT_DEFAULT 30 /* 30 seconds */
-#define GFARM_GFMD_RECONNECTION_TIMEOUT_DEFAULT 30 /* 30 seconds */
 #define GFARM_ATTR_CACHE_LIMIT_DEFAULT		40000 /* 40,000 entries */
 #define GFARM_ATTR_CACHE_TIMEOUT_DEFAULT	1000 /* 1,000 milli second */
 #define GFARM_SCHEDULE_CACHE_TIMEOUT_DEFAULT 600 /* 10 minutes */
@@ -361,14 +363,13 @@ char *gfarm_localfs_datadir = NULL;
 #define GFARM_SCHEDULE_VIRTUAL_LOAD_DEFAULT	0.3F
 #define GFARM_SCHEDULE_WRITE_LOCAL_PRIORITY_DEFAULT 1 /* enable */
 #define GFARM_MINIMUM_FREE_DISK_SPACE_DEFAULT	(128 * 1024 * 1024) /* 128MB */
+#define GFARM_SIMULTANEOUS_REPLICATION_RECEIVERS_DEFAULT	20
 #define GFARM_GFSD_CONNECTION_CACHE_DEFAULT 16 /* 16 free connections */
 #define GFARM_GFMD_CONNECTION_CACHE_DEFAULT  8 /*  8 free connections */
 #define GFARM_RECORD_ATIME_DEFAULT 1 /* enable */
 #define MISC_DEFAULT -1
 int gfarm_log_level = MISC_DEFAULT;
 int gfarm_log_message_verbose = MISC_DEFAULT;
-int gfarm_no_file_system_node_timeout = MISC_DEFAULT;
-int gfarm_gfmd_reconnection_timeout = MISC_DEFAULT;
 int gfarm_attr_cache_limit = MISC_DEFAULT;
 int gfarm_attr_cache_timeout = MISC_DEFAULT;
 int gfarm_schedule_cache_timeout = MISC_DEFAULT;
@@ -378,6 +379,7 @@ float gfarm_schedule_virtual_load = MISC_DEFAULT;
 static char *schedule_write_target_domain = NULL;
 static int schedule_write_local_priority = MISC_DEFAULT;
 gfarm_int64_t gfarm_minimum_free_disk_space = MISC_DEFAULT;
+int gfarm_simultaneous_replication_receivers = MISC_DEFAULT;
 int gfarm_gfsd_connection_cache = MISC_DEFAULT;
 int gfarm_gfmd_connection_cache = MISC_DEFAULT;
 int gfarm_metadb_stack_size = MISC_DEFAULT;
@@ -537,8 +539,12 @@ gfarm_strtoken(char **cursorp, char **tokenp)
 			for (;;) {
 				if (*s == '\'')
 					break;
-				if (*s == '\0' || *s == '\n')
+				if (*s == '\0' || *s == '\n') {
+					gflog_debug(GFARM_MSG_1000919,
+						"Unterminated single quote "
+						"found in string");
 					return (GFARM_ERRMSG_UNTERMINATED_SINGLE_QUOTE);
+				}
 				*p++ = *s++;
 			}
 			s++;
@@ -548,11 +554,20 @@ gfarm_strtoken(char **cursorp, char **tokenp)
 			for (;;) {
 				if (*s == '"')
 					break;
-				if (*s == '\0' || *s == '\n')
+				if (*s == '\0' || *s == '\n') {
+					gflog_debug(GFARM_MSG_1000920,
+						"Unterminated double quote "
+						"found in string");
 					return (GFARM_ERRMSG_UNTERMINATED_DOUBLE_QUOTE);
+				}
 				if (*s == '\\') {
-					if (s[1] == '\0' || s[1] == '\n')
+					if (s[1] == '\0' || s[1] == '\n') {
+						gflog_debug(GFARM_MSG_1000921,
+							"Unterminated double "
+							"quote found in string"
+						);
 						return (GFARM_ERRMSG_UNTERMINATED_DOUBLE_QUOTE);
+					}
 					/*
 					 * only interpret `\"' and `\\'
 					 * in double quotation.
@@ -566,8 +581,11 @@ gfarm_strtoken(char **cursorp, char **tokenp)
 			break;
 		case '\\':
 			s++;
-			if (*s == '\0' || *s == '\n')
+			if (*s == '\0' || *s == '\n') {
+				gflog_debug(GFARM_MSG_1000922,
+					"Incomplete escape found in string");
 				return (GFARM_ERRMSG_INCOMPLETE_ESCAPE);
+			}
 			*p++ = *s++;
 			break;
 		case '\n':	
@@ -601,16 +619,30 @@ parse_auth_arguments(char *p, char **op)
 	/* assert(strcmp(*op, "auth") == 0); */
 
 	e = gfarm_strtoken(&p, &command);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000923,
+			"parsing of auth command argument (%s) failed: %s",
+			p, gfarm_error_string(e));
 		return (e);
-	if (command == NULL)
+	}
+	if (command == NULL) {
+		gflog_debug(GFARM_MSG_1000924,
+			"Missing first auth command argument");
 		return (GFARM_ERRMSG_MISSING_1ST_AUTH_COMMAND_ARGUMENT);
+	}
 
 	e = gfarm_strtoken(&p, &auth);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000925,
+			"parsing of auth method argument (%s) failed: %s",
+			p, gfarm_error_string(e));
 		return (e);
-	if (auth == NULL)
+	}
+	if (auth == NULL) {
+		gflog_debug(GFARM_MSG_1000926,
+			"Missing second auth method argument");
 		return (GFARM_ERRMSG_MISSING_2ND_AUTH_METHOD_ARGUMENT);
+	}
 	if (strcmp(auth, "*") == 0 || strcmp(auth, "ALL") == 0) {
 		auth_method = GFARM_AUTH_METHOD_ALL;
 	} else {
@@ -619,20 +651,37 @@ parse_auth_arguments(char *p, char **op)
 			*op = "2nd(auth-method) argument";
 			if (e == GFARM_ERR_NO_SUCH_OBJECT)
 				e = GFARM_ERRMSG_UNKNOWN_AUTH_METHOD;
+			gflog_debug(GFARM_MSG_1000927,
+				"parsing of auth method (%s) failed: %s",
+				auth, gfarm_error_string(e));
 			return (e);
 		}
 	}
 
 	e = gfarm_strtoken(&p, &host);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000928,
+			"parsing of auth host argument (%s) failed: %s",
+			p, gfarm_error_string(e));
 		return (e);
-	if (host == NULL)
+	}
+	if (host == NULL) {
+		gflog_debug(GFARM_MSG_1000929,
+			"Missing third auth host spec argument");
 		return (GFARM_ERRMSG_MISSING_3RD_HOST_SPEC_ARGUMENT);
+	}
 	e = gfarm_strtoken(&p, &tmp);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000930,
+			"parsing of auth arguments (%s) failed: %s",
+			p, gfarm_error_string(e));
 		return (e);
-	if (tmp != NULL)
+	}
+	if (tmp != NULL) {
+		gflog_debug(GFARM_MSG_1000931,
+			"Too many auth arguments passed");
 		return (GFARM_ERRMSG_TOO_MANY_ARGUMENTS);
+	}
 	e = gfarm_hostspec_parse(host, &hostspecp);
 	if (e != GFARM_ERR_NO_ERROR) {
 		/*
@@ -640,6 +689,9 @@ parse_auth_arguments(char *p, char **op)
 		 * because it may be too long.
 		 */
 		*op = "3rd(host-spec) argument";
+		gflog_debug(GFARM_MSG_1000932,
+			"parsing of auth host spec (%s) failed: %s",
+			host, gfarm_error_string(e));
 		return (e);
 	}
 
@@ -654,10 +706,18 @@ parse_auth_arguments(char *p, char **op)
 		 */
 		*op = "1st(auth-command) argument";
 		gfarm_hostspec_free(hostspecp);
+		gflog_debug(GFARM_MSG_1000933,
+			"Unknown auth subcommand (%s)",
+			command);
 		return (GFARM_ERRMSG_UNKNOWN_AUTH_SUBCOMMAND);
 	}
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000934,
+			"Failed to enable/disable auth (%s)(%s)(%s): (%s)",
+			command, auth, host,
+			gfarm_error_string(e));
 		gfarm_hostspec_free(hostspecp);
+	}
 	return (e);
 }
 
@@ -671,20 +731,36 @@ parse_netparam_arguments(char *p, char **op)
 	/* assert(strcmp(*op, "netparam") == 0); */
 
 	e = gfarm_strtoken(&p, &option);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000935,
+			"parsing of netparam option argument (%s) failed: %s",
+			p, gfarm_error_string(e));
 		return (e);
-	if (option == NULL)
+	}
+	if (option == NULL) {
+		gflog_debug(GFARM_MSG_1000936,
+			"Missing first netparam option argument");
 		return (GFARM_ERRMSG_MISSING_NETPARAM_OPTION_ARGUMENT);
+	}
 
 	e = gfarm_strtoken(&p, &host);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000937,
+			"parsing of netparam host argument (%s) failed: %s",
+			p, gfarm_error_string(e));
 		return (e);
+	}
 	if (host == NULL) {
 		/* if 2nd argument is omitted, it is treated as "*". */
 		host = "*";
 	} else if ((e = gfarm_strtoken(&p, &tmp)) != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000938,
+			"parsing of netparam arguments (%s) failed: %s",
+			p, gfarm_error_string(e));
 		return (e);
 	} else if (tmp != NULL) {
+		gflog_debug(GFARM_MSG_1000939,
+			"Too many netparam arguments passed");
 		return (GFARM_ERRMSG_TOO_MANY_ARGUMENTS);
 	}
 	
@@ -695,6 +771,9 @@ parse_netparam_arguments(char *p, char **op)
 		 * because it may be too long.
 		 */
 		*op = "2nd(host-spec) argument";
+		gflog_debug(GFARM_MSG_1000940,
+			"parsing of netparam host spec (%s) failed: %s",
+			host, gfarm_error_string(e));
 		return (e);
 	}
 
@@ -706,6 +785,9 @@ parse_netparam_arguments(char *p, char **op)
 		 */
 		*op = "1st(sockopt-option) argument";
 		gfarm_hostspec_free(hostspecp);
+		gflog_debug(GFARM_MSG_1000941,
+			"add netparam config (%s)(%s) failed: %s",
+			host, option, gfarm_error_string(e));
 		return (e);
 	}
 	return (GFARM_ERR_NO_ERROR);
@@ -722,14 +804,25 @@ parse_sockopt_arguments(char *p, char **op)
 	/* assert(strcmp(*op, "sockopt") == 0); */
 
 	e = gfarm_strtoken(&p, &option);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000942,
+			"parsing of sockopt option argument (%s) failed: %s",
+			p, gfarm_error_string(e));
 		return (e);
-	if (option == NULL)
+	}
+	if (option == NULL) {
+		gflog_debug(GFARM_MSG_1000943,
+			"Missing sockopt option argument");
 		return (GFARM_ERRMSG_MISSING_SOCKOPT_OPTION_ARGUMENT);
+	}
 
 	e = gfarm_strtoken(&p, &host);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000944,
+			"parsing of sockopt host argument (%s) failed: %s",
+			p, gfarm_error_string(e));
 		return (e);
+	}
 	if (host == NULL) {
 		/*
 		 * if 2nd argument is omitted, it is treated as:
@@ -738,10 +831,17 @@ parse_sockopt_arguments(char *p, char **op)
 		is_listener = 1;
 	} else {
 		is_listener = strcmp(host, "LISTENER") == 0;
-		if ((e = gfarm_strtoken(&p, &tmp)) != GFARM_ERR_NO_ERROR)
+		if ((e = gfarm_strtoken(&p, &tmp)) != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1000945,
+				"parsing of sockopt arguments (%s) failed: %s",
+				p, gfarm_error_string(e));
 			return (e);
-		if (tmp != NULL)
+		}
+		if (tmp != NULL) {
+			gflog_debug(GFARM_MSG_1000946,
+				"Too many sockopt arguments passed");
 			return (GFARM_ERRMSG_TOO_MANY_ARGUMENTS);
+		}
 	}
 	
 	if (is_listener) {
@@ -752,6 +852,10 @@ parse_sockopt_arguments(char *p, char **op)
 			 * because it may be too long.
 			 */
 			*op = "1st(sockopt-option) argument";
+			gflog_debug(GFARM_MSG_1000947,
+				"add sockopt listener config failed"
+				"(%s)(%s): %s",
+				host, option, gfarm_error_string(e));
 			return (e);
 		}
 	}
@@ -764,6 +868,9 @@ parse_sockopt_arguments(char *p, char **op)
 			 * because it may be too long.
 			 */
 			*op = "2nd(host-spec) argument";
+			gflog_debug(GFARM_MSG_1000948,
+				"parsing of sockopt host (%s) failed: %s",
+				host, gfarm_error_string(e));
 			return (e);
 		}
 
@@ -775,6 +882,9 @@ parse_sockopt_arguments(char *p, char **op)
 			 */
 			*op = "1st(sockopt-option) argument";
 			gfarm_hostspec_free(hostspecp);
+			gflog_debug(GFARM_MSG_1000949,
+				"add sockopt config (%s)(%s) failed: %s",
+				host, option, gfarm_error_string(e));
 			return (e);
 		}
 	}
@@ -833,18 +943,37 @@ parse_local_user_map(char *p, char **op)
 	char *tmp, *mapfile;
 
 	e = gfarm_strtoken(&p, &mapfile);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000950,
+			"parsing of local user map "
+			"mapfile argument (%s) failed: %s",
+			p, gfarm_error_string(e));
 		return (e);
-	if (mapfile == NULL)
+	}
+	if (mapfile == NULL) {
+		gflog_debug(GFARM_MSG_1000951,
+			"Missing local user map mapfile argument");
 		return (GFARM_ERRMSG_MISSING_USER_MAP_FILE_ARGUMENT);
+	}
 	e = gfarm_strtoken(&p, &tmp);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000952,
+			"parsing of local user map arguments (%s) failed: %s",
+			p, gfarm_error_string(e));
 		return (e);
-	if (tmp != NULL)
+	}
+	if (tmp != NULL) {
+		gflog_debug(GFARM_MSG_1000953,
+			"Too many local user map arguments passed");
 		return (GFARM_ERRMSG_TOO_MANY_ARGUMENTS);
+	}
 	mapfile = strdup(mapfile);
-	if (mapfile == NULL)
+	if (mapfile == NULL) {
+		gflog_debug(GFARM_MSG_1000954,
+			"allocation of 'mapfile' failed: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
 		return (GFARM_ERR_NO_MEMORY);
+	}
 	e = gfarm_stringlist_add(&local_user_map_file_list, mapfile);
 	return (e);
 }
@@ -918,15 +1047,29 @@ get_one_argument(char *p, char **rv)
 	char *tmp, *s;
 
 	e = gfarm_strtoken(&p, &s);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000955,
+			"parsing of one argument (%s) failed: %s",
+			p, gfarm_error_string(e));
 		return (e);
-	if (s == NULL)
+	}
+	if (s == NULL) {
+		gflog_debug(GFARM_MSG_1000956,
+			"Missing argument");
 		return (GFARM_ERRMSG_MISSING_ARGUMENT);
+	}
 	e = gfarm_strtoken(&p, &tmp);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000957,
+			"parsing of arguments (%s) failed: %s",
+			p, gfarm_error_string(e));
 		return (e);
-	if (tmp != NULL)
+	}
+	if (tmp != NULL) {
+		gflog_debug(GFARM_MSG_1000958,
+			"Too many arguments passed");
 		return (GFARM_ERRMSG_TOO_MANY_ARGUMENTS);
+	}
 
 	*rv = s;
 	return (GFARM_ERR_NO_ERROR);
@@ -939,14 +1082,23 @@ parse_set_var(char *p, char **rv)
 	char *s;
 
 	e = get_one_argument(p, &s);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000959,
+			"get_one_argument failed "
+			"when parsing var (%s): %s",
+			p, gfarm_error_string(e));
 		return (e);
+	}
 
 	if (*rv != NULL) /* first line has precedence */
 		return (GFARM_ERR_NO_ERROR);
 	s = strdup(s);
-	if (s == NULL)
+	if (s == NULL) {
+		gflog_debug(GFARM_MSG_1000960,
+			"allocation of argument failed when parsing set var: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
 		return (GFARM_ERR_NO_MEMORY);
+	}
 	*rv = s;
 	return (GFARM_ERR_NO_ERROR);
 }
@@ -959,19 +1111,39 @@ parse_set_misc_int(char *p, int *vp)
 	int v;
 
 	e = get_one_argument(p, &s);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000961,
+			"get_one_argument failed "
+			"when parsing misc integer (%s): %s",
+			p, gfarm_error_string(e));
 		return (e);
+	}
 
 	if (*vp != MISC_DEFAULT) /* first line has precedence */
 		return (GFARM_ERR_NO_ERROR);
 	errno = 0;
 	v = strtol(s, &ep, 10);
-	if (errno != 0)
-		return (gfarm_errno_to_error(errno));
-	if (ep == s)
+	if (errno != 0) {
+		int save_errno = errno;
+		gflog_debug(GFARM_MSG_1000962,
+			"conversion to integer failed "
+			"when parsing misc integer (%s): %s",
+			p, strerror(save_errno));
+		return (gfarm_errno_to_error(save_errno));
+	}
+	if (ep == s) {
+		gflog_debug(GFARM_MSG_1000963,
+			"Integer expected when parsing misc integer but (%s)",
+			s);
 		return (GFARM_ERRMSG_INTEGER_EXPECTED);
-	if (*ep != '\0')
+	}
+	if (*ep != '\0') {
+		gflog_debug(GFARM_MSG_1000964,
+			"Invalid character found "
+			"when parsing misc integer (%s)",
+			s);
 		return (GFARM_ERRMSG_INVALID_CHARACTER);
+	}
 	*vp = v;
 	return (GFARM_ERR_NO_ERROR);
 }
@@ -1009,17 +1181,32 @@ parse_set_misc_offset(char *p, gfarm_off_t *vp)
 	gfarm_off_t v;
 
 	e = get_one_argument(p, &s);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000965,
+			"get_one_argument failed "
+			"when parsing misc offset (%s): %s",
+			p, gfarm_error_string(e));
 		return (e);
+	}
 
 	if (*vp != MISC_DEFAULT) /* first line has precedence */
 		return (GFARM_ERR_NO_ERROR);
 	errno = 0;
 	v = gfarm_strtoi64(s, &ep);
-	if (errno != 0)
-		return (gfarm_errno_to_error(errno));
-	if (ep == s)
+	if (errno != 0) {
+		int save_errno = errno;
+		gflog_debug(GFARM_MSG_1000966,
+			"conversion to int64 failed "
+			"when parsing misc offset (%s): %s",
+			p, strerror(save_errno));
+		return (gfarm_errno_to_error(save_errno));
+	}
+	if (ep == s) {
+		gflog_debug(GFARM_MSG_1000967,
+			"Integer expected when parsing misc offset but (%s)",
+			s);
 		return (GFARM_ERRMSG_INTEGER_EXPECTED);
+	}
 	if (*ep != '\0') {
 		switch (*ep) {
 		case 'k': case 'K': ep++; v *= 1024; break;
@@ -1027,8 +1214,13 @@ parse_set_misc_offset(char *p, gfarm_off_t *vp)
 		case 'g': case 'G': ep++; v *= 1024 * 1024 * 1024; break;
 		case 't': case 'T': ep++; v *=1024*1024; v *=1024*1024; break;
 		}
-		if (*ep != '\0')
+		if (*ep != '\0') {
+			gflog_debug(GFARM_MSG_1000968,
+				"Invalid character found "
+				"when parsing misc offset (%s)",
+				s);
 			return (GFARM_ERRMSG_INVALID_CHARACTER);
+		}
 	}
 	*vp = v;
 	return (GFARM_ERR_NO_ERROR);
@@ -1042,8 +1234,13 @@ parse_set_misc_enabled(char *p, int *vp)
 	int v;
 
 	e = get_one_argument(p, &s);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000969,
+			"get_one_argument failed "
+			"when parsing misc enabled (%s): %s",
+			p, gfarm_error_string(e));
 		return (e);
+	}
 
 	if (*vp != MISC_DEFAULT) /* first line has precedence */
 		return (GFARM_ERR_NO_ERROR);
@@ -1051,8 +1248,13 @@ parse_set_misc_enabled(char *p, int *vp)
 		v = 1;
 	else if (strcmp(s, "disable") == 0)
 		v = 0;
-	else
+	else {
+		gflog_debug(GFARM_MSG_1000970,
+			"'enable' or 'disable' expected "
+			"when parsing misc enabled but (%s)",
+			s);
 		return (GFARM_ERRMSG_ENABLED_OR_DISABLED_EXPECTED);
+	}
 	*vp = v;
 	return (GFARM_ERR_NO_ERROR);
 }
@@ -1065,8 +1267,13 @@ parse_cred_config(char *p, char *service,
 	char *s;
 
 	e = get_one_argument(p, &s);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000971,
+			"get_one_argument failed "
+			"when parsing cred config (%s): %s",
+			p, gfarm_error_string(e));
 		return (e);
+	}
 
 	return ((*set)(service, s));
 }
@@ -1079,14 +1286,23 @@ parse_log_level(char *p, int *vp)
 	int v;
 
 	e = get_one_argument(p, &s);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1000972,
+			"get_one_argument failed "
+			"when parsing log level (%s): %s",
+			p, gfarm_error_string(e));
 		return (e);
+	}
 
 	if (*vp != MISC_DEFAULT) /* first line has precedence */
 		return (GFARM_ERR_NO_ERROR);
 	v = gflog_syslog_name_to_priority(s);
-	if (v == -1)
+	if (v == -1) {
+		gflog_debug(GFARM_MSG_1000973,
+			"Invalid syslog priority level (%s)",
+			s);
 		return (GFARM_ERRMSG_INVALID_SYSLOG_PRIORITY_LEVEL);
+	}
 	*vp = v;
 	return (GFARM_ERR_NO_ERROR);
 }
@@ -1220,10 +1436,6 @@ parse_one_line(char *s, char *p, char **op)
 	} else if (strcmp(s, o = "log_message_verbose_level") == 0) {
 		e = parse_set_misc_int(p, &gfarm_log_message_verbose);
 		gflog_set_message_verbose(gfarm_log_message_verbose);
-	} else if (strcmp(s, o = "no_file_system_node_timeout") == 0) {
-		e = parse_set_misc_int(p, &gfarm_no_file_system_node_timeout);
-	} else if (strcmp(s, o = "gfmd_reconnection_timeout") == 0) {
-		e = parse_set_misc_int(p, &gfarm_gfmd_reconnection_timeout);
 	} else if (strcmp(s, o = "attr_cache_limit") == 0) {
 		e = parse_set_misc_int(p, &gfarm_attr_cache_limit);
 	} else if (strcmp(s, o = "attr_cache_timeout") == 0) {
@@ -1242,6 +1454,9 @@ parse_one_line(char *s, char *p, char **op)
 		e = parse_set_var(p, &schedule_write_target_domain);
 	} else if (strcmp(s, o = "minimum_free_disk_space") == 0) {
 		e = parse_set_misc_offset(p, &gfarm_minimum_free_disk_space);
+	} else if (strcmp(s, o = "simultaneous_replication_receivers") == 0) {
+		e = parse_set_misc_int(p,
+		    &gfarm_simultaneous_replication_receivers);
 	} else if (strcmp(s, o = "gfsd_connection_cache") == 0) {
 		e = parse_set_misc_int(p, &gfarm_gfsd_connection_cache);
 	} else if (strcmp(s, o = "gfmd_connection_cache") == 0) {
@@ -1261,6 +1476,10 @@ parse_one_line(char *s, char *p, char **op)
 
 	} else {
 		o = s;
+		gflog_debug(GFARM_MSG_1000974,
+			"Unknown keyword encountered "
+			"when parsing one line (%s)",
+			s);
 		e = GFARM_ERRMSG_UNKNOWN_KEYWORD;
 	}
 	*op = o;
@@ -1315,6 +1534,10 @@ gfarm_config_read_file(FILE *config, int *lineno_p)
 		if (e != GFARM_ERR_NO_ERROR) {
 			fclose(config);
 			*lineno_p = lineno;
+			gflog_debug(GFARM_MSG_1000975,
+				"parsing token failed"
+				"when reading config file (%s): %s",
+				p, gfarm_error_string(e));
 			return (e);
 		}
 	}
@@ -1355,12 +1578,6 @@ gfarm_config_set_default_misc(void)
 		gfarm_log_message_verbose = GFARM_LOG_MESSAGE_VERBOSE_DEFAULT;
 	gflog_set_message_verbose(gfarm_log_message_verbose);
 
-	if (gfarm_no_file_system_node_timeout == MISC_DEFAULT)
-		gfarm_no_file_system_node_timeout =
-		    GFARM_NO_FILE_SYSTEM_NODE_TIMEOUT_DEFAULT;
-	if (gfarm_gfmd_reconnection_timeout == MISC_DEFAULT)
-		gfarm_gfmd_reconnection_timeout =
-		    GFARM_GFMD_RECONNECTION_TIMEOUT_DEFAULT;
 	if (gfarm_attr_cache_limit == MISC_DEFAULT)
 		gfarm_attr_cache_limit = GFARM_ATTR_CACHE_LIMIT_DEFAULT;
 	if (gfarm_attr_cache_timeout == MISC_DEFAULT)
@@ -1381,6 +1598,9 @@ gfarm_config_set_default_misc(void)
 	if (gfarm_minimum_free_disk_space == MISC_DEFAULT)
 		gfarm_minimum_free_disk_space =
 		    GFARM_MINIMUM_FREE_DISK_SPACE_DEFAULT;
+	if (gfarm_simultaneous_replication_receivers == MISC_DEFAULT)
+		 gfarm_simultaneous_replication_receivers =
+		    GFARM_SIMULTANEOUS_REPLICATION_RECEIVERS_DEFAULT;
 	if (gfarm_gfsd_connection_cache == MISC_DEFAULT)
 		gfarm_gfsd_connection_cache =
 		    GFARM_GFSD_CONNECTION_CACHE_DEFAULT;
