@@ -147,15 +147,15 @@ enum gfm_proto_command {
 
 	GFM_PROTO_REOPEN,
 	GFM_PROTO_CLOSE_READ,
-	GFM_PROTO_CLOSE_WRITE,
+	GFM_PROTO_CLOSE_WRITE,			/* for COMPAT_GFARM_2_3 */
 	GFM_PROTO_LOCK,
 	GFM_PROTO_TRYLOCK,
 	GFM_PROTO_UNLOCK,
 	GFM_PROTO_LOCK_INFO,
-	GFM_PROTO_SWITCH_BACK_CHANNEL,
-	GFM_PROTO_FILE_OP_RESERVE8,
-	GFM_PROTO_FILE_OP_RESERVE9,
-	GFM_PROTO_FILE_OP_RESERVE10,
+	GFM_PROTO_SWITCH_BACK_CHANNEL,		/* for COMPAT_GFARM_2_3 */
+	GFM_PROTO_SWITCH_ASYNC_BACK_CHANNEL,
+	GFM_PROTO_CLOSE_WRITE_V2_4,
+	GFM_PROTO_GENERATION_UPDATED,
 	GFM_PROTO_FILE_OP_RESERVE11,
 	GFM_PROTO_FILE_OP_RESERVE12,
 	GFM_PROTO_FILE_OP_RESERVE13,
@@ -221,9 +221,9 @@ enum gfm_proto_command {
 	GFM_PROTO_REPLICA_LIST_BY_HOST,
 	GFM_PROTO_REPLICA_REMOVE_BY_HOST,
 	GFM_PROTO_REPLICA_REMOVE_BY_FILE,
-	GFM_PROTO_REPLICA_OP_RESERVE4,
-	GFM_PROTO_REPLICA_OP_RESERVE5,
-	GFM_PROTO_REPLICA_OP_RESERVE6,
+	GFM_PROTO_REPLICA_INFO_GET,
+	GFM_PROTO_REPLICATE_FILE_FROM_TO,
+	GFM_PROTO_REPLICATE_FILE_TO,
 	GFM_PROTO_REPLICA_OP_RESERVE7,
 	GFM_PROTO_REPLICA_OP_RESERVE8,
 	GFM_PROTO_REPLICA_OP_RESERVE9,
@@ -236,12 +236,12 @@ enum gfm_proto_command {
 
 	/* replica management from gfsd */
 
-	GFM_PROTO_REPLICA_ADDING,
-	GFM_PROTO_REPLICA_ADDED,
+	GFM_PROTO_REPLICA_ADDING,		/* for COMPAT_GFARM_2_3 */
+	GFM_PROTO_REPLICA_ADDED,		/* for COMPAT_GFARM_2_3 */
 	GFM_PROTO_REPLICA_REMOVE,
-	GFM_PROTO_REPLICA_ADD,
-	GFM_PROTO_REPLICA_ADDED2,
-	GFM_PROTO_REPLICA_MNG_RESERVE5,
+	GFM_PROTO_REPLICA_ADD,			/* not used? */
+	GFM_PROTO_REPLICA_ADDED2,		/* for COMPAT_GFARM_2_3 */
+	GFM_PROTO_REPLICATION_RESULT,
 	GFM_PROTO_REPLICA_MNG_RESERVE6,
 	GFM_PROTO_REPLICA_MNG_RESERVE7,
 	GFM_PROTO_REPLICA_MNG_RESERVE8,
@@ -351,6 +351,12 @@ enum gfm_proto_command {
 #define GFM_PROTO_SCHED_FLAG_RTT_AVAIL		4 /* always FALSE for now */
 #define GFM_PROTO_LOADAVG_FSCALE 		2048
 
+/* output of GFM_PROTO_CLOSE_WRITE_V2_4 */
+#define	GFM_PROTO_CLOSE_WRITE_GENERATION_UPDATE_NEEDED	1
+
+/* output of GFM_PROTO_REPLICA_INFO_GET */
+#define GFM_PROTO_REPLICA_FLAG_INCOMPLETE	1
+#define GFM_PROTO_REPLICA_FLAG_DEAD_HOST	2
 
 #if 0 /* There isn't gfm_proto.c for now. */
 extern char GFM_SERVICE_TAG[];
