@@ -216,8 +216,13 @@ gfs_client_connect_unix(struct sockaddr *peer_addr, int *sockp)
 		gfs_client_connection_gc(); /* XXX FIXME: GC all descriptors */
 		sock = socket(PF_UNIX, SOCK_STREAM, 0);
 	}
-	if (sock == -1)
-		return (gfarm_errno_to_error(errno));
+	if (sock == -1) {
+		save_errno = errno;
+		gflog_debug(GFARM_MSG_1001172,
+			"creation of UNIX socket failed: %s",
+			strerror(save_errno));
+		return (gfarm_errno_to_error(save_errno));
+	}
 
 	memset(&peer_un, 0, sizeof(peer_un));
 	/* XXX inet_ntoa() is not MT-safe on some platforms */
@@ -234,8 +239,12 @@ gfs_client_connect_unix(struct sockaddr *peer_addr, int *sockp)
 	if (rv == -1) {
 		save_errno = errno;
 		close(sock);
-		if (save_errno != ENOENT)
+		if (save_errno != ENOENT) {
+			gflog_debug(GFARM_MSG_1001173,
+				"connect() with UNIX socket failed: %s",
+				strerror(save_errno));
 			return (gfarm_errno_to_error(save_errno));
+		}
 
 		/* older gfsd doesn't support UNIX connection, try INET */
 		sock = -1;
@@ -257,8 +266,13 @@ gfs_client_connect_inet(const char *canonical_hostname,
 		gfs_client_connection_gc(); /* XXX FIXME: GC all descriptors */
 		sock = socket(PF_INET, SOCK_STREAM, 0);
 	}
-	if (sock == -1)
-		return (gfarm_errno_to_error(errno));
+	if (sock == -1) {
+		save_errno = errno;
+		gflog_debug(GFARM_MSG_1001174,
+			"Creation of inet socket failed: %s",
+			strerror(save_errno));
+		return (gfarm_errno_to_error(save_errno));
+	}
 
 	/* XXX - how to report setsockopt(2) failure ? */
 	gfarm_sockopt_apply_by_name_addr(sock, canonical_hostname, peer_addr);
@@ -273,6 +287,10 @@ gfs_client_connect_inet(const char *canonical_hostname,
 		e = gfarm_bind_source_ip(sock, source_ip);
 		if (e != GFARM_ERR_NO_ERROR) {
 			close(sock);
+			gflog_debug(GFARM_MSG_1001175,
+				"bind() with inet socket (%s) failed: %s",
+				source_ip,
+				gfarm_error_string(e));
 			return (e);
 		}
 	}
@@ -282,6 +300,9 @@ gfs_client_connect_inet(const char *canonical_hostname,
 		if (errno != EINPROGRESS) {
 			save_errno = errno;
 			close(sock);
+			gflog_debug(GFARM_MSG_1001176,
+				"connect() with inet socket failed: %s",
+				strerror(save_errno));
 			return (gfarm_errno_to_error(save_errno));
 		}
 		*connection_in_progress_p = 1;
@@ -308,34 +329,51 @@ gfs_client_connection_alloc(const char *canonical_hostname,
 #endif
 	if (sockaddr_is_local(peer_addr)) {
 		e = gfs_client_connect_unix(peer_addr, &sock);
-		if (e != GFARM_ERR_NO_ERROR)
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001177,
+				"connect with unix socket failed: %s",
+				gfarm_error_string(e));
 			return (e);
+		}
 		connection_in_progress = 0;
 		is_local = 1;
 	}
 	if (sock == -1) {
 		e = gfs_client_connect_inet(canonical_hostname,
 		    peer_addr, &connection_in_progress, &sock, source_ip);
-		if (e != GFARM_ERR_NO_ERROR)
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001178,
+				"connect with inet socket failed: %s",
+				gfarm_error_string(e));
 			return (e);
+		}
 	}
 	fcntl(sock, F_SETFD, 1); /* automatically close() on exec(2) */
 
 	GFARM_MALLOC(gfs_server);
 	if (gfs_server == NULL) {
 		close(sock);
+		gflog_debug(GFARM_MSG_1001179,
+			"allocation of 'gfs_server' failed: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
 		return (GFARM_ERR_NO_MEMORY);
 	}
 	e = gfp_xdr_new_socket(sock, &gfs_server->conn);
 	if (e != GFARM_ERR_NO_ERROR) {
 		free(gfs_server);
 		close(sock);
+		gflog_debug(GFARM_MSG_1001180,
+			"gfp_xdr_new_socket() failed: %s",
+			gfarm_error_string(e));
 		return (e);
 	}
 	gfs_server->hostname = strdup(canonical_hostname);
 	if (gfs_server->hostname == NULL) {
 		e = gfp_xdr_free(gfs_server->conn);
 		free(gfs_server);
+		gflog_debug(GFARM_MSG_1001181,
+			"allocation of 'gfs_server->hostname' failed: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
 		return (GFARM_ERR_NO_MEMORY);
 	}
 	gfs_server->port = ntohs(((struct sockaddr_in *)peer_addr)->sin_port);
@@ -363,8 +401,12 @@ gfs_client_connection_alloc_and_auth(const char *canonical_hostname,
 
 	e = gfs_client_connection_alloc(canonical_hostname, peer_addr,
 	    cache_entry, &connection_in_progress, &gfs_server, source_ip);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001182,
+			"gfs_client_connection_alloc() failed: %s",
+			gfarm_error_string(e));
 		return (e);
+	}
 	if (connection_in_progress)
 		e = gfarm_connect_wait(gfp_xdr_fd(gfs_server->conn),
 		    GFS_CLIENT_CONNECT_TIMEOUT);
@@ -378,6 +420,9 @@ gfs_client_connection_alloc_and_auth(const char *canonical_hostname,
 		free(gfs_server->hostname);
 		gfp_xdr_free(gfs_server->conn);
 		free(gfs_server);
+		gflog_debug(GFARM_MSG_1001183,
+			"connection or authentication failed: %s",
+			gfarm_error_string(e));
 	}
 	return (e);
 }
@@ -430,8 +475,13 @@ gfs_client_connection_acquire(const char *canonical_hostname,
 	    canonical_hostname,
 	    ntohs(((struct sockaddr_in *)peer_addr)->sin_port),
 	    &cache_entry, &created);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001184,
+			"acquirement of cached connection (%s) failed: %s",
+			canonical_hostname,
+			gfarm_error_string(e));
 		return (e);
+	}
 	if (!created) {
 		*gfs_serverp = gfp_cached_connection_get_data(cache_entry);
 		return (GFARM_ERR_NO_ERROR);
@@ -442,6 +492,9 @@ gfs_client_connection_acquire(const char *canonical_hostname,
 		gfp_cached_connection_purge_from_cache(&gfs_server_cache,
 		    cache_entry);
 		gfp_uncached_connection_dispose(cache_entry);
+		gflog_debug(GFARM_MSG_1001185,
+			"allocation or authentication failed: %s",
+			gfarm_error_string(e));
 	}
 	return (e);
 }
@@ -462,8 +515,13 @@ gfs_client_connection_acquire_by_host(struct gfm_connection *gfm_server,
 	 */
 	e = gfp_cached_connection_acquire(&gfs_server_cache,
 	    canonical_hostname, port, &cache_entry, &created);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001186,
+			"acquirement of cached connection (%s) failed: %s",
+			canonical_hostname,
+			gfarm_error_string(e));
 		return (e);
+	}
 	if (!created) {
 		*gfs_serverp = gfp_cached_connection_get_data(cache_entry);
 		return (GFARM_ERR_NO_ERROR);
@@ -477,6 +535,9 @@ gfs_client_connection_acquire_by_host(struct gfm_connection *gfm_server,
 		gfp_cached_connection_purge_from_cache(&gfs_server_cache,
 		    cache_entry);
 		gfp_uncached_connection_dispose(cache_entry);
+		gflog_debug(GFARM_MSG_1001187,
+			"client authentication failed: %s",
+			gfarm_error_string(e));
 	}
 	return (e);
 }
@@ -496,12 +557,20 @@ gfs_client_connect(const char *canonical_hostname, struct sockaddr *peer_addr,
 	struct gfp_cached_connection *cache_entry;
 
 	e = gfp_uncached_connection_new(&cache_entry);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001188,
+			"making new uncached connection failed: %s",
+			gfarm_error_string(e));
 		return (e);
+	}
 	e = gfs_client_connection_alloc_and_auth(canonical_hostname, peer_addr,
 	    cache_entry, gfs_serverp, NULL);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
 		gfp_uncached_connection_dispose(cache_entry);
+		gflog_debug(GFARM_MSG_1001189,
+			"client authentication failed: %s",
+			gfarm_error_string(e));
+	}
 	return (e);
 }
 
@@ -579,6 +648,9 @@ gfs_client_connect_start_auth(int events, int fd, void *closure,
 			return;
 		}
 	}
+	gflog_debug(GFARM_MSG_1001190,
+		"starting client connection auth failed: %s",
+		gfarm_error_string(state->error));
 	if (state->continuation != NULL)
 		(*state->continuation)(state->closure);
 }
@@ -599,12 +671,19 @@ gfs_client_connect_request_multiplexed(struct gfarm_eventqueue *q,
 	/* clone of gfs_client_connect() */
 
 	GFARM_MALLOC(state);
-	if (state == NULL)
+	if (state == NULL) {
+		gflog_debug(GFARM_MSG_1001191,
+			"allocation of client connect state failed: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
 		return (GFARM_ERR_NO_MEMORY);
+	}
 
 	e = gfp_uncached_connection_new(&cache_entry);
 	if (e != GFARM_ERR_NO_ERROR) {
 		free(state);
+		gflog_debug(GFARM_MSG_1001192,
+			"making new uncached connection failed: %s",
+			gfarm_error_string(e));
 		return (e);
 	}
 	e = gfs_client_connection_alloc(canonical_hostname, peer_addr,
@@ -612,6 +691,9 @@ gfs_client_connect_request_multiplexed(struct gfarm_eventqueue *q,
 	if (e != GFARM_ERR_NO_ERROR) {
 		gfp_uncached_connection_dispose(cache_entry);
 		free(state);
+		gflog_debug(GFARM_MSG_1001193,
+			"allocation of client connection failed: %s",
+			gfarm_error_string(e));
 		return (e);
 	}
 
@@ -656,6 +738,9 @@ gfs_client_connect_request_multiplexed(struct gfarm_eventqueue *q,
 	}
 	free(state);
 	gfs_client_connection_dispose(gfs_server);
+	gflog_debug(GFARM_MSG_1001194,
+		"request for multiplexed client connect failed: %s",
+		gfarm_error_string(e));
 	return (e);
 }
 
@@ -672,6 +757,9 @@ gfs_client_connect_result_multiplexed(
 	free(state);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gfs_client_connection_dispose(gfs_server);
+		gflog_debug(GFARM_MSG_1001195,
+			"error in result of multiplexed client connect: %s",
+			gfarm_error_string(e));
 		return (e);
 	}
 
@@ -793,6 +881,11 @@ gfs_client_rpc_request(struct gfs_connection *gfs_server, int command,
 		gfs_client_execute_hook_for_connection_error(gfs_server);
 		gfs_client_purge_from_cache(gfs_server);
 	}
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001196,
+			"gfp_xdr_vrpc_request() failed: %s",
+			gfarm_error_string(e));
+	}
 	return (e);
 }
 
@@ -811,8 +904,12 @@ gfs_client_rpc_result(struct gfs_connection *gfs_server, int just,
 		gfs_client_execute_hook_for_connection_error(gfs_server);
 		gfs_client_purge_from_cache(gfs_server);
 	}
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001197,
+			"gfp_xdr_flush() failed : %s",
+			gfarm_error_string(e));
 		return (e);
+	}
 
 	va_start(ap, format);
 	e = gfp_xdr_vrpc_result(gfs_server->conn, just,
@@ -823,13 +920,20 @@ gfs_client_rpc_result(struct gfs_connection *gfs_server, int just,
 		gfs_client_execute_hook_for_connection_error(gfs_server);
 		gfs_client_purge_from_cache(gfs_server);
 	}
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001198,
+			"gfp_xdr_vrpc_result() failed: %s",
+			gfarm_error_string(e));
 		return (e);
+	}
 	if (errcode != 0) {
 		/*
 		 * We just use gfarm_error_t as the errcode,
 		 * Note that GFARM_ERR_NO_ERROR == 0.
 		 */
+		gflog_debug(GFARM_MSG_1001199,
+			"gfp_xdr_vrpc_result() failed errcode=%d",
+			errcode);
 		return (errcode);
 	}
 	return (GFARM_ERR_NO_ERROR);
@@ -854,13 +958,20 @@ gfs_client_rpc(struct gfs_connection *gfs_server, int just, int command,
 		gfs_client_execute_hook_for_connection_error(gfs_server);
 		gfs_client_purge_from_cache(gfs_server);
 	}
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001200,
+			"gfp_xdr_vrpc() failed: %s",
+			gfarm_error_string(e));
 		return (e);
+	}
 	if (errcode != 0) {
 		/*
 		 * We just use gfarm_error_t as the errcode,
 		 * Note that GFARM_ERR_NO_ERROR == 0.
 		 */
+		gflog_debug(GFARM_MSG_1001201,
+			"gfp_xdr_vrpc() errcode=%d",
+			errcode);
 		return (errcode);
 	}
 	return (GFARM_ERR_NO_ERROR);
@@ -876,6 +987,10 @@ gfs_client_process_set(struct gfs_connection *gfs_server,
 	    type, size, key, pid);
 	if (e == GFARM_ERR_NO_ERROR)
 		gfs_server->pid = pid;
+	else
+		gflog_debug(GFARM_MSG_1001202,
+			"gfs_client_rpc() failed: %s",
+			gfarm_error_string(e));
 	return (e);
 }
 
@@ -887,6 +1002,10 @@ gfs_client_open(struct gfs_connection *gfs_server, gfarm_int32_t fd)
 	e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_OPEN, "i/", fd);
 	if (e == GFARM_ERR_NO_ERROR)
 		++gfs_server->opened;
+	else
+		gflog_debug(GFARM_MSG_1001203,
+			"gfs_client_rpc() failed: %s",
+			gfarm_error_string(e));
 	return (e);
 }
 
@@ -898,21 +1017,37 @@ gfs_client_open_local(struct gfs_connection *gfs_server, gfarm_int32_t fd,
 	int rv, local_fd;
 	gfarm_int8_t dummy; /* needs at least 1 byte */
 
-	if (!gfs_server->is_local)
+	if (!gfs_server->is_local) {
+		gflog_debug(GFARM_MSG_1001204,
+			"gfs server is local: %s",
+			gfarm_error_string(GFARM_ERR_OPERATION_NOT_SUPPORTED));
 		return (GFARM_ERR_OPERATION_NOT_SUPPORTED);
+	}
 
 	/* we have to set `just' flag here */
 	e = gfs_client_rpc(gfs_server, 1, GFS_PROTO_OPEN_LOCAL, "i/", fd);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001205,
+			"gfs_client_rpc() failed: %s",
+			gfarm_error_string(e));
 		return (e);
+	}
 
 	/* layering violation, but... */
 	rv = gfarm_fd_receive_message(gfp_xdr_fd(gfs_server->conn),
 	    &dummy, sizeof(dummy), 1, &local_fd);
-	if (rv == -1) /* EOF */
+	if (rv == -1) { /* EOF */
+		gflog_debug(GFARM_MSG_1001206,
+			"Unexpected EOF when receiving message: %s",
+			gfarm_error_string(GFARM_ERR_UNEXPECTED_EOF));
 		return (GFARM_ERR_UNEXPECTED_EOF);
-	if (rv != 0)
+	}
+	if (rv != 0) {
+		gflog_debug(GFARM_MSG_1001207,
+			"receiving message failed: %s",
+			gfarm_error_string(gfarm_errno_to_error(rv)));
 		return (gfarm_errno_to_error(rv));
+	}
 	/* both `dummy' and `local_fd` are passed by using host byte order. */
 	*fd_ret = local_fd;
 	++gfs_server->opened;
@@ -927,6 +1062,10 @@ gfs_client_close(struct gfs_connection *gfs_server, gfarm_int32_t fd)
 	e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_CLOSE, "i/", fd);
 	if (e == GFARM_ERR_NO_ERROR)
 		--gfs_server->opened;
+	else
+		gflog_debug(GFARM_MSG_1001208,
+			"gfs_client_rpc() failed: %s",
+			gfarm_error_string(e));
 	return (e);
 }
 
@@ -939,10 +1078,18 @@ gfs_client_pread(struct gfs_connection *gfs_server,
 
 	if ((e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_PREAD, "iil/b",
 	    fd, (int)size, off,
-	    size, np, buffer)) != GFARM_ERR_NO_ERROR)
+	    size, np, buffer)) != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001209,
+			"gfs_client_rpc() failed: %s",
+			gfarm_error_string(e));
 		return (e);
-	if (*np > size)
+	}
+	if (*np > size) {
+		gflog_debug(GFARM_MSG_1001210,
+			"Protocol error in client pread (%llu)>(%llu)",
+			(unsigned long long)*np, (unsigned long long)size);
 		return (GFARM_ERRMSG_GFS_PROTO_PREAD_PROTOCOL);
+	}
 	return (GFARM_ERR_NO_ERROR);
 }
 
@@ -956,11 +1103,19 @@ gfs_client_pwrite(struct gfs_connection *gfs_server,
 	gfarm_int32_t n; /* size_t may be 64bit */
 
 	if ((e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_PWRITE, "ibl/i",
-	    fd, size, buffer, off, &n)) != GFARM_ERR_NO_ERROR)
+	    fd, size, buffer, off, &n)) != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001211,
+			"gfs_client_rpc() failed: %s",
+			gfarm_error_string(e));
 		return (e);
+	}
 	*np = n;
-	if (n > size)
+	if (n > size) {
+		gflog_debug(GFARM_MSG_1001212,
+			"Protocol error in client pwrite (%llu)>(%llu)",
+			(unsigned long long)*np, (unsigned long long)size);
 		return (GFARM_ERRMSG_GFS_PROTO_PWRITE_PROTOCOL);
+	}
 	return (GFARM_ERR_NO_ERROR);
 }
 
@@ -1102,6 +1257,9 @@ gfs_client_statfs_send_request(int events, int fd, void *closure,
 			state->error = gfarm_errno_to_error(rv);
 		}
 	}
+	gflog_debug(GFARM_MSG_1001213,
+		"request for client statfs failed: %s",
+		gfarm_error_string(state->error));
 	if (state->continuation != NULL)
 		(*state->continuation)(state->closure);
 }
@@ -1139,6 +1297,9 @@ gfs_client_statfs_request_multiplexed(struct gfarm_eventqueue *q,
 	GFARM_MALLOC(state);
 	if (state == NULL) {
 		e = GFARM_ERR_NO_MEMORY;
+		gflog_debug(GFARM_MSG_1001214,
+			"allocation of client statfs state failed: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
 		goto error_return;
 	}
 
@@ -1153,6 +1314,9 @@ gfs_client_statfs_request_multiplexed(struct gfarm_eventqueue *q,
 	    gfs_client_statfs_send_request, state);
 	if (state->writable == NULL) {
 		e = GFARM_ERR_NO_MEMORY;
+		gflog_debug(GFARM_MSG_1001215,
+			"allocation of state->writable failed: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
 		goto error_free_state;
 	}
 	/*
@@ -1166,12 +1330,18 @@ gfs_client_statfs_request_multiplexed(struct gfarm_eventqueue *q,
 	    gfs_client_statfs_recv_result, state);
 	if (state->readable == NULL) {
 		e = GFARM_ERR_NO_MEMORY;
+		gflog_debug(GFARM_MSG_1001216,
+			"allocation of state->readable failed: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
 		goto error_free_writable;
 	}
 	/* go to gfs_client_statfs_send_request() */
 	rv = gfarm_eventqueue_add_event(q, state->writable, NULL);
 	if (rv != 0) {
 		e = gfarm_errno_to_error(rv);
+		gflog_debug(GFARM_MSG_1001217,
+			"adding event to event queue failed: %s",
+			gfarm_error_string(e));
 		goto error_free_readable;
 	}
 	*statepp = state;
@@ -1232,8 +1402,12 @@ gfs_client_replica_recv(struct gfs_connection *gfs_server,
 		gfs_client_execute_hook_for_connection_error(gfs_server);
 		gfs_client_purge_from_cache(gfs_server);
 	}
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001218,
+			"gfs_request_client_rpc() failed: %s",
+			gfarm_error_string(e));
 		return (e);
+	}
 
 	for (;;) {
 		gfarm_int32_t size;
@@ -1306,6 +1480,11 @@ gfs_client_replica_recv(struct gfs_connection *gfs_server,
 		e = e_write;
 	if (e == GFARM_ERR_NO_ERROR)
 		e = e_rpc;
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001219,
+			"receiving client replica failed: %s",
+			gfarm_error_string(e));
+	}
 	return (e);
 }
 
@@ -1410,14 +1589,23 @@ gfs_client_command_request(struct gfs_connection *gfs_server,
 		}
 		GFARM_MALLOC_ARRAY(xdisplay_name_cache,
 			strlen(prefix) + strlen(dpy) + 1);
-		if (xdisplay_name_cache == NULL)
+		if (xdisplay_name_cache == NULL) {
+			gflog_debug(GFARM_MSG_1001220,
+				"allocation of array 'xdisplay_name_cache' "
+				"failed: %s",
+				gfarm_error_string(GFARM_ERR_NO_MEMORY));
 			return (GFARM_ERR_NO_MEMORY);
+		}
 		sprintf(xdisplay_name_cache, "%s%s", prefix, dpy);
 		GFARM_MALLOC_ARRAY(xdisplay_env_cache,
 		    sizeof(xdisplay_env_format) + strlen(xdisplay_name_cache));
 		if (xdisplay_env_cache == NULL) {
 			free(xdisplay_name_cache);
 			xdisplay_name_cache = NULL;
+			gflog_debug(GFARM_MSG_1001221,
+				"allocation of array 'xdisplay_env_cache' "
+				"failed : %s",
+				gfarm_error_string(GFARM_ERR_NO_MEMORY));
 			return (GFARM_ERR_NO_MEMORY);
 		}
 		sprintf(xdisplay_env_cache, xdisplay_env_format,
@@ -1440,12 +1628,20 @@ gfs_client_command_request(struct gfs_connection *gfs_server,
 			sizeof(xauth_command_format) +
 			strlen(XAUTH_COMMAND) +
 			strlen(xdisplay_name_cache));
-		if (xauth_command == NULL)
+		if (xauth_command == NULL) {
+			gflog_debug(GFARM_MSG_1001222,
+				"allocation of array 'xauth_command' "
+				"failed : %s",
+				gfarm_error_string(GFARM_ERR_NO_MEMORY));
 			return (GFARM_ERR_NO_MEMORY);
+		}
 		sprintf(xauth_command, xauth_command_format,
 			XAUTH_COMMAND, xdisplay_name_cache);
 		if ((fp = popen(xauth_command, "r")) == NULL) {
 			free(xauth_command);
+			gflog_debug(GFARM_MSG_1001223,
+				"popen() for auth command pipe failed: %s",
+				gfarm_error_string(GFARM_ERR_NO_MEMORY));
 			return (GFARM_ERR_NO_MEMORY);
 		}
 		s = fgets(line, sizeof line, fp);
@@ -1455,6 +1651,11 @@ gfs_client_command_request(struct gfs_connection *gfs_server,
 			xauth_cache = strdup(line);
 			if (xauth_cache == NULL) {
 				free(xdisplay_name_cache);
+				gflog_debug(GFARM_MSG_1001224,
+					"allocation of 'xauth_cache' "
+					"failed : %s",
+					gfarm_error_string(
+						GFARM_ERR_NO_MEMORY));
 				return (GFARM_ERR_NO_MEMORY);
 			}
 		}
@@ -1481,53 +1682,87 @@ gfs_client_command_request(struct gfs_connection *gfs_server,
 		((flags & GFS_CLIENT_COMMAND_FLAG_SHELL_COMMAND) ?
 		GFS_CLIENT_COMMAND_FLAG_SHELL_COMMAND : 0) |
 		(xauth_copy ? GFS_CLIENT_COMMAND_FLAG_XAUTHCOPY : 0));
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001225,
+			"gfs_request_client_rpc() failed: %s",
+			gfarm_error_string(e));
 		return (e);
+	}
 	/*
 	 * argv
 	 */
 	for (i = 0; i < na; i++) {
 		e = gfp_xdr_send(gfs_server->conn, "s", argv[i]);
-		if (e != GFARM_ERR_NO_ERROR)
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001226,
+				"sending argv[%d] (%s) failed: %s",
+				i, argv[i],
+				gfarm_error_string(e));
 			return (e);
+		}
 	}
 	/*
 	 * envp
 	 */
 	for (i = 0; i < ne; i++) {
 		e = gfp_xdr_send(gfs_server->conn, "s", envp[i]);
-		if (e != GFARM_ERR_NO_ERROR)
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001227,
+				"sending envp[%d] (%s) failed: %s",
+				i, envp[i],
+				gfarm_error_string(e));
 			return (e);
+		}
 	}
 	if (xenv_copy) {
 		e = gfp_xdr_send(gfs_server->conn, "s", xdisplay_env_cache);
-		if (e != GFARM_ERR_NO_ERROR)
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001228,
+				"sending xdisplay_env_cache failed: %s",
+				gfarm_error_string(e));
 			return (e);
+		}
 	}
 	if (xauth_copy) {
 		/*
 		 * xauth
 		 */
 		e = gfp_xdr_send(gfs_server->conn, "s", xauth_cache);
-		if (e != GFARM_ERR_NO_ERROR)
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001229,
+				"sending xauth_cache failed: %s",
+				gfarm_error_string(e));
 			return (e);
+		}
 	}
 	/* we have to set `just' flag here */
 	e = gfs_client_rpc_result(gfs_server, 1, "i", &pid);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001230,
+			"gfs_client_rpc_result() failed: %s",
+			gfarm_error_string(e));
 		return (e);
+	}
 
 	gfs_server->context = GFARM_MALLOC(cc);
-	if (gfs_server->context == NULL)
+	if (gfs_server->context == NULL) {
+		gflog_debug(GFARM_MSG_1001231,
+			"allocation of 'gfs_server->context' failed: %s",
+			gfarm_error_string(GFARM_ERR_NO_MEMORY));
 		return (GFARM_ERR_NO_MEMORY);
+	}
 
 	/*
 	 * Now, we set the connection file descriptor non-blocking mode.
 	 */
 	if (fcntl(conn_fd, F_SETFL, O_NONBLOCK) == -1) {
+		int save_errno = errno;
 		free(gfs_server->context);
 		gfs_server->context = NULL;
-		return (gfarm_errno_to_error(errno));
+		gflog_debug(GFARM_MSG_1001232,
+			"setting conn_fd to non-blocking mode failed: %s",
+			strerror(save_errno));
+		return (gfarm_errno_to_error(save_errno));
 	}
 
 	/*
@@ -1686,6 +1921,9 @@ gfs_client_command_io_fd_set(struct gfs_connection *gfs_server,
 					e = GFARM_ERRMSG_GFSD_ABORTED;
 				cc->server_state =
 					GFS_COMMAND_SERVER_STATE_ABORTED;
+				gflog_debug(GFARM_MSG_1001233,
+					"gfp_xdr_recv() failed: %s",
+					gfarm_error_string(e));
 				return (e);
 			}
 			switch (cmd) {
@@ -1701,12 +1939,18 @@ gfs_client_command_io_fd_set(struct gfs_connection *gfs_server,
 						e = GFARM_ERRMSG_GFSD_ABORTED;
 					cc->server_state =
 					    GFS_COMMAND_SERVER_STATE_ABORTED;
+					gflog_debug(GFARM_MSG_1001234,
+						"gfp_xdr_recv() failed: "
+						"%s",
+						gfarm_error_string(e));
 					return (e);
 				}
 				if (fd != FDESC_STDOUT && fd != FDESC_STDERR) {
 					/* XXX - something wrong */
 					cc->server_state =
 					    GFS_COMMAND_SERVER_STATE_ABORTED;
+					gflog_debug(GFARM_MSG_1001235,
+						"Illegal file descriptor");
 					return (GFARM_ERRMSG_GFSD_DESCRIPTOR_ILLEGAL);
 				}
 				if (len <= 0) {
@@ -1724,6 +1968,8 @@ gfs_client_command_io_fd_set(struct gfs_connection *gfs_server,
 				/* XXX - something wrong */
 				cc->server_state =
 				    GFS_COMMAND_SERVER_STATE_ABORTED;
+				gflog_debug(GFARM_MSG_1001236,
+					"Unknown reply %d", cmd);
 				return (GFARM_ERRMSG_GFSD_REPLY_UNKNOWN);
 			}
 		} else if (cc->server_state==GFS_COMMAND_SERVER_STATE_OUTPUT) {
@@ -1743,12 +1989,17 @@ gfs_client_command_io_fd_set(struct gfs_connection *gfs_server,
 				    GFARM_ERR_NO_ERROR);
 				cc->server_state =
 					GFS_COMMAND_SERVER_STATE_ABORTED;
+				gflog_debug(GFARM_MSG_1001237,
+					"read operation on iobuffer failed: %s",
+					gfarm_error_string(e));
 				return (e);
 			}
 			if (gfarm_iobuffer_is_read_eof(
 					cc->iobuffer[cc->server_output_fd])) {
 				cc->server_state =
 					GFS_COMMAND_SERVER_STATE_ABORTED;
+				gflog_debug(GFARM_MSG_1001238,
+					"read operation on iobuffer aborted");
 				return (GFARM_ERRMSG_GFSD_ABORTED);
 			}
 		}
@@ -1765,6 +2016,9 @@ gfs_client_command_io_fd_set(struct gfs_connection *gfs_server,
 				    != GFARM_ERR_NO_ERROR) {
 					cc->server_state =
 					    GFS_COMMAND_SERVER_STATE_ABORTED;
+					gflog_debug(GFARM_MSG_1001239,
+						"sending signal failed: %s",
+						gfarm_error_string(e));
 					return (e);
 				}
 			} else if (gfarm_iobuffer_is_writable(
@@ -1785,6 +2039,9 @@ gfs_client_command_io_fd_set(struct gfs_connection *gfs_server,
 				    != GFARM_ERR_NO_ERROR) {
 					cc->server_state =
 					    GFS_COMMAND_SERVER_STATE_ABORTED;
+					gflog_debug(GFARM_MSG_1001240,
+						"sending fd falied: %s",
+						gfarm_error_string(e));
 					return (e);
 				}
 				cc->client_state =
@@ -1804,6 +2061,10 @@ gfs_client_command_io_fd_set(struct gfs_connection *gfs_server,
 				gfarm_iobuffer_set_error(
 				    cc->iobuffer[FDESC_STDIN],
 				    GFARM_ERR_NO_ERROR);
+				gflog_debug(GFARM_MSG_1001241,
+					"write operation on iobuffer "
+					"failed: %s",
+					gfarm_error_string(e));
 				return (e);
 			}
 		}
@@ -1877,8 +2138,12 @@ gfs_client_command_send_signal(struct gfs_connection *gfs_server, int sig)
 	while (gfs_client_command_is_running(gfs_server) &&
 	       cc->pending_signal != 0) {
 		e = gfs_client_command_io(gfs_server, NULL);
-		if (e != GFARM_ERR_NO_ERROR)
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001242,
+				"gfs_client_command_io() failed: %s",
+				gfarm_error_string(e));
 			return (e);
+		}
 	}
 	if (!gfs_client_command_is_running(gfs_server))
 		return (gfarm_errno_to_error(ESRCH));
@@ -1964,7 +2229,11 @@ gfs_client_command_result(struct gfs_connection *gfs_server,
 	 * Now, we recover the connection file descriptor blocking mode.
 	 */
 	if (fcntl(gfp_xdr_fd(gfs_server->conn), F_SETFL, 0) == -1) {
-		return (gfarm_errno_to_error(errno));
+		int save_errno = errno;
+		gflog_debug(GFARM_MSG_1001243,
+			"setting conn fd to blocking mode failed: %s",
+			strerror(save_errno));
+		return (gfarm_errno_to_error(save_errno));
 	}
 	return (gfs_client_rpc(gfs_server, 0, GFS_PROTO_COMMAND_EXIT_STATUS,
 			       "/iii",
@@ -1981,12 +2250,25 @@ gfs_client_command(struct gfs_connection *gfs_server,
 
 	e = gfs_client_command_request(gfs_server, path, argv, envp, flags,
 				       &pid);
-	if (e)
+	if (e) {
+		gflog_debug(GFARM_MSG_1001244,
+			"gfs_client_command_request() failed: %s",
+			gfarm_error_string(e));
 		return (e);
+	}
 	while (gfs_client_command_is_running(gfs_server))
 		e = gfs_client_command_io(gfs_server, NULL);
 	e2 = gfs_client_command_result(gfs_server,
 				       term_signal, exit_status, exit_flag);
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001245,
+			"gfs_client_command_io() failed: %s",
+			gfarm_error_string(e));
+	} else if (e2 != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001246,
+			"gfs_client_command_result() failed: %s",
+			gfarm_error_string(e2));
+	}
 	return (e != GFARM_ERR_NO_ERROR ? e : e2);
 }
 
@@ -2019,8 +2301,13 @@ gfs_client_get_load_request(int sock,
 		rv = sendto(sock, &command, sizeof(command), 0,
 		    server_addr, server_addr_size);
 	}
-	if (rv == -1)
-		return (gfarm_errno_to_error(errno));
+	if (rv == -1) {
+		int save_errno = errno;
+		gflog_debug(GFARM_MSG_1001247,
+			"write or send operation on socket failed: %s",
+			strerror(save_errno));
+		return (gfarm_errno_to_error(save_errno));
+	}
 	return (GFARM_ERR_NO_ERROR);
 }
 
@@ -2050,8 +2337,13 @@ gfs_client_get_load_result(int sock,
 		rv = recvfrom(sock, nloadavg, sizeof(nloadavg), 0,
 		    server_addr, server_addr_sizep);
 	}
-	if (rv == -1)
-		return (gfarm_errno_to_error(errno));
+	if (rv == -1) {
+		int save_errno = errno;
+		gflog_debug(GFARM_MSG_1001248,
+			"read or receive operation from socket failed: %s",
+			strerror(save_errno));
+		return (gfarm_errno_to_error(save_errno));
+	}
 
 #ifndef WORDS_BIGENDIAN
 	swab(&nloadavg[0], &loadavg[0], sizeof(loadavg[0]));
@@ -2165,6 +2457,9 @@ gfs_client_get_load_request_multiplexed(struct gfarm_eventqueue *q,
 	GFARM_MALLOC(state);
 	if (state == NULL) {
 		e = GFARM_ERR_NO_MEMORY;
+		gflog_debug(GFARM_MSG_1001249,
+			"allocation of client get load state failed: %s",
+			gfarm_error_string(e));
 		goto error_close_sock;
 	}
 
@@ -2173,6 +2468,10 @@ gfs_client_get_load_request_multiplexed(struct gfarm_eventqueue *q,
 	    gfs_client_get_load_send, state);
 	if (state->writable == NULL) {
 		e = GFARM_ERR_NO_MEMORY;
+		gflog_debug(GFARM_MSG_1001250,
+			"allocation of client get load state->writable "
+			"failed: %s",
+			gfarm_error_string(e));
 		goto error_free_state;
 	}
 	/*
@@ -2185,12 +2484,19 @@ gfs_client_get_load_request_multiplexed(struct gfarm_eventqueue *q,
 	    gfs_client_get_load_receive, state);
 	if (state->readable == NULL) {
 		e = GFARM_ERR_NO_MEMORY;
+		gflog_debug(GFARM_MSG_1001251,
+			"allocation of client get load state->readable "
+			"failed: %s",
+			gfarm_error_string(e));
 		goto error_free_writable;
 	}
 	/* go to gfs_client_get_load_send() */
 	rv = gfarm_eventqueue_add_event(q, state->writable, NULL);
 	if (rv != 0) {
 		e = gfarm_errno_to_error(rv);
+		gflog_debug(GFARM_MSG_1001252,
+			"adding event to event queue failed: %s",
+			gfarm_error_string(e));
 		goto error_free_readable;
 	}
 
@@ -2428,8 +2734,12 @@ gfs_client_apply_all_hosts(
 	struct gfarm_host_info *hosts;
 
 	e = gfarm_host_info_get_all(&nhosts, &hosts);
-	if (e != NULL)
+	if (e != NULL) {
+		gflog_debug(GFARM_MSG_1001253,
+			"gfarm_host_info_get_all() failed: %s",
+			gfarm_error_string(e));
 		return (e);
+	}
 
         j = 0;
 	*nhosts_succeed = 0;
