@@ -361,8 +361,11 @@ search_idle_network_list_init(struct gfm_connection *gfm_server)
 	}
 
 	GFARM_MALLOC(net);
-	if (net == NULL)
+	if (net == NULL) {
+		gflog_debug(GFARM_MSG_1001425,
+		    "search_idle_network_list_init: no memory");
 		return (GFARM_ERR_NO_MEMORY);
+	}
 	net->rtt_usec = 0; /* i.e. local network */
 #if 0 /* XXX not implemented yet */
 	/*
@@ -465,8 +468,11 @@ search_idle_network_list_add(struct sockaddr *addr,
 	}
 	/* first host in the network */
 	GFARM_MALLOC(net);
-	if (net == NULL)
+	if (net == NULL) {
+		gflog_debug(GFARM_MSG_1001426,
+		    "search_idle_network_list_add: no memory");
 		return (GFARM_ERR_NO_MEMORY);
+	}
 	net->min = min;
 	net->max = max;
 	net->flags = 0;
@@ -489,8 +495,12 @@ search_idle_host_state_initialize(struct gfm_connection *gfm_server)
 
 	e = gfp_conn_hash_table_init(&search_idle_hosts_state,
 	    HOSTS_HASHTAB_SIZE);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001427,
+		    "search_idle_host_state_initialize: hash_table_init: %s",
+		    gfarm_error_string(e));
 		return (e);
+	}
 
 	search_idle_network_list_init(gfm_server); /* ignore any error here */
 
@@ -514,15 +524,25 @@ search_idle_host_state_add_host_sched_info(struct gfm_connection *gfm_server,
 
 	if (search_idle_hosts_state == NULL) {
 		e = search_idle_host_state_initialize(gfm_server);
-		if (e != GFARM_ERR_NO_ERROR)
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001428,
+			    "search_idle_host_state_add_host_sched_info: "
+			    "search_idle_host_state_initialize: %s",
+			    gfarm_error_string(e));
 			return (e);
+		}
 	}
 
 	e = gfp_conn_hash_enter(&search_idle_hosts_state, HOSTS_HASHTAB_SIZE,
 	    sizeof(*h), hostname, info->port, gfm_client_username(gfm_server),
 	    &entry, &created);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001429,
+		    "search_idle_host_state_add_host_sched_info: "
+		    "gfp_conn_hash_enter: %s",
+		    gfarm_error_string(e));
 		return (e);
+	}
 
 	h = gfarm_hash_entry_data(entry);
 	if (created || (h->flags & HOST_STATE_FLAG_ADDR_AVAIL) == 0 ||
@@ -542,8 +562,14 @@ search_idle_host_state_add_host_sched_info(struct gfm_connection *gfm_server,
 			h->scheduled = 0;
 			h->flags = 0;
 		} else if ((h->flags & HOST_STATE_FLAG_ADDR_AVAIL) == 0) {
-			if (!is_expired(&h->addr_cache_time, ADDR_EXPIRATION))
+			if (!is_expired(&h->addr_cache_time,
+				ADDR_EXPIRATION)) {
+				gflog_debug(GFARM_MSG_1001430,
+				    "search_idle_host_state_"
+				    "add_host_sched_info: %s: unknown host",
+				    hostname);
 				return (GFARM_ERR_UNKNOWN_HOST);
+			}
 		} else { /* cope with address change */
 			assert(is_expired(&h->addr_cache_time, ADDR_EXPIRATION)
 			    );
@@ -553,19 +579,33 @@ search_idle_host_state_add_host_sched_info(struct gfm_connection *gfm_server,
 		e = gfm_host_address_get(gfm_server,
 		    hostname, h->port, &h->addr, NULL);
 		gettimeofday(&h->addr_cache_time, NULL);
-		if (e != GFARM_ERR_NO_ERROR)
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001431,
+			    "search_idle_host_state_add_host_sched_info: "
+			    "gfm_host_address_get(%s): %s",
+			    hostname, gfarm_error_string(e));
 			return (e);
+		}
 		h->flags |= HOST_STATE_FLAG_ADDR_AVAIL;
 		e = search_idle_network_list_add(&h->addr, &h->net);
 		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001432,
+			    "search_idle_host_state_add_host_sched_info: "
+			    "search_idle_network_list_add: %s",
+			    gfarm_error_string(e));
 			h->net = NULL;
 			return (e);
 		}
 	} else if (h->net == NULL) {
 		/* search_idle_network_list_add() failed at the last time */
 		e = search_idle_network_list_add(&h->addr, &h->net);
-		if (e != GFARM_ERR_NO_ERROR)
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001433,
+			    "search_idle_host_state_add_host_sched_info: "
+			    "retrying search_idle_network_list_add: %s",
+			    gfarm_error_string(e));
 			return (e);
+		}
 	}
 
 	*hp = h;
@@ -588,8 +628,13 @@ gfarm_schedule_host_cache_clear_auth(struct gfs_connection *gfs_server)
 	    gfs_client_port(gfs_server),
 	    gfs_client_username(gfs_server),
 	    &entry);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001434,
+		    "gfarm_schedule_host_cache_clear_auth: "
+		    "connection is not cached: %s",
+		    gfarm_error_string(e));
 		return (e);
+	}
 
 	h = gfarm_hash_entry_data(entry);
 	h->flags &= ~HOST_STATE_FLAG_AUTH_SUCCEED;
@@ -639,8 +684,13 @@ search_idle_candidate_list_reset(struct gfm_connection *gfm_server,
 
 	if (search_idle_hosts_state == NULL) {
 		gfarm_error_t e = search_idle_host_state_initialize(gfm_server);
-		if (e != GFARM_ERR_NO_ERROR)
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001435,
+			    "search_idle_candidate_list_reset: "
+			    "search_idle_host_state_initialize: %s",
+			    gfarm_error_string(e));
 			return (e);
+		}
 	}
 
 	search_idle_candidate_host_number = 0;
@@ -737,6 +787,10 @@ search_idle_candidate_list_add(struct gfm_connection *gfm_server,
 	if (e != GFARM_ERR_NO_ERROR) {
 		if (e == GFARM_ERR_UNKNOWN_HOST) /* just ignore this host */
 			return (GFARM_ERR_NO_ERROR);
+		gflog_debug(GFARM_MSG_1001436,
+		    "search_idle_candidate_list_add: "
+		    "search_idle_host_state_add_host_sched_info: %s",
+		    gfarm_error_string(e));
 		return (e);
 	}
 	if ((h->flags & HOST_STATE_FLAG_SCHEDULING) != 0) {
@@ -847,12 +901,18 @@ search_idle_init_state(struct search_idle_state *s, int desired_hosts,
 	 * otherwise GFARM_ERRMSG_NO_FILESYSTEM_NODE.
 	 */
 	if (s->enough_number == 0 || s->desired_number == 0 ||
-	    search_idle_candidate_list == NULL)
+	    search_idle_candidate_list == NULL) {
+		gflog_debug(GFARM_MSG_1001437,
+		    "search_idle_init_state: no answer is requested");
 		return (GFARM_ERRMSG_NO_FILESYSTEM_NODE);
+	}
 
 	s->q = gfarm_eventqueue_alloc();
-	if (s->q == NULL)
+	if (s->q == NULL) {
+		gflog_debug(GFARM_MSG_1001438,
+		    "search_idle_init_state: no memory");
 		return (GFARM_ERR_NO_MEMORY);
+	}
 	s->available_hosts_number = s->usable_hosts_number =
 	    s->idle_hosts_number = s->semi_idle_hosts_number = 0;
 	s->concurrency = 0;
@@ -973,6 +1033,12 @@ search_idle_statfs_callback(void *closure)
 		h->diskavail = bavail * bsize;
 		search_idle_record(c); /* completed */
 	}
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001439,
+		    "search_idle_statfs_callback: "
+		    "gfs_client_statfs_result_multiplexed: %s",
+		    gfarm_error_string(e));
+	}
 	gfs_client_connection_free(c->gfs_server);
 	c->state->concurrency--;
 	h->net->ongoing--;
@@ -1015,9 +1081,18 @@ search_idle_connect_callback(void *closure)
 			}
 			/* failed to request */
 			gfs_client_connection_free(c->gfs_server);
+			gflog_debug(GFARM_MSG_1001440,
+			    "search_idle_connect_callback: "
+			    "gfs_client_statfs_request_multiplexed: %s",
+			    gfarm_error_string(e));
 #if 0 /* We always see disk free space */
 		}
 #endif
+	} else {
+		gflog_debug(GFARM_MSG_1001441,
+		    "search_idle_connect_callback: "
+		    "gfs_client_connect_result_multiplexed: %s",
+		    gfarm_error_string(e));
 	}
 	c->state->concurrency--;
 	c->h->net->ongoing--;
@@ -1068,7 +1143,16 @@ search_idle_load_callback(void *closure)
 				return; /* request continues */
 			}
 			/* failed to connect */
+			gflog_debug(GFARM_MSG_1001442,
+			    "search_idle_load_callback: "
+			    "gfs_client_connect_request_multiplexed: %s",
+			    gfarm_error_string(e));
 		}
+	} else {
+		gflog_debug(GFARM_MSG_1001443,
+		    "search_idle_load_callback: "
+		    "gfs_client_get_load_result_multiplexed: %s",
+		    gfarm_error_string(e));
 	}
 	c->state->concurrency--;
 	c->h->net->ongoing--;
@@ -1199,13 +1283,20 @@ search_idle_try_host(struct search_idle_state *s,
 	while (s->concurrency >= CONCURRENCY) {
 		rv = gfarm_eventqueue_turn(s->q, NULL);
 		/* XXX - how to report this error? */
-		if (rv != 0 && rv != EAGAIN && rv != EINTR)
+		if (rv != 0 && rv != EAGAIN && rv != EINTR) {
+			gflog_debug(GFARM_MSG_1001444,
+			    "search_idle_try_host: gfarm_eventqueue_turn: %s",
+			    strerror(rv));
 			return (gfarm_errno_to_error(rv));
+		}
 	}
 
 	GFARM_MALLOC(c);
-	if (c == NULL)
+	if (c == NULL) {
+		gflog_debug(GFARM_MSG_1001445,
+		    "search_idle_try_host: no memory");
 		return (GFARM_ERR_NO_MEMORY);
+	}
 	c->state = s;
 	c->h = h;
 	c->do_record = do_record;
@@ -1217,6 +1308,10 @@ search_idle_try_host(struct search_idle_state *s,
 	    search_idle_load_callback, c,
 	    &gls);
 	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001446,
+		    "search_idle_try_host: "
+		    "gfs_client_get_load_request_multiplexed: %s",
+		    gfarm_error_string(e));
 		free(c);
 		return (GFARM_ERR_NO_ERROR); /* We won't report the error */
 	}
@@ -1347,8 +1442,11 @@ search_idle_by_rtt_order(struct search_idle_state *s)
 		return (GFARM_ERR_NO_ERROR);
 
 	GFARM_MALLOC_ARRAY(netarray, nnets);
-	if (netarray == NULL)
+	if (netarray == NULL) {
+		gflog_debug(GFARM_MSG_1001447,
+		    "search_idle_by_rtt_order: no memory");
 		return (GFARM_ERR_NO_MEMORY);
+	}
 	i = 0;
 	for (net = search_idle_network_list; net != NULL; net = net->next) {
 		if (net == search_idle_local_net) /* already searched */
@@ -1394,8 +1492,12 @@ search_idle(int *nohostsp, char **ohosts, int *oports, int write_mode)
 	gfs_profile(gfarm_gettimerval(&t1));
 	e = search_idle_init_state(&s, *nohostsp, default_search_method,
 	    write_mode);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001448,
+		    "search_idle: search_idle_init_state: %s",
+		    gfarm_error_string(e));
 		return (e);
+	}
 	gfs_profile(gfarm_gettimerval(&t2));
 
 	/*
@@ -1420,15 +1522,24 @@ search_idle(int *nohostsp, char **ohosts, int *oports, int write_mode)
 			   gfarm_timerval_sub(&t4, &t3)));
 
 	gfarm_eventqueue_free(s.q);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001449,
+		    "search_idle: search_idle_by_rtt_order: %s",
+		    gfarm_error_string(e));
 		return (e);
-	if (s.usable_hosts_number == 0)
+	}
+	if (s.usable_hosts_number == 0) {
+		gflog_debug(GFARM_MSG_1001450,
+		    "search_idle: no filesystem node");
 		return (GFARM_ERRMSG_NO_FILESYSTEM_NODE);
+	}
 
 	assert(s.available_hosts_number >= s.usable_hosts_number);
 	GFARM_MALLOC_ARRAY(results, s.available_hosts_number);
-	if (results == NULL)
+	if (results == NULL) {
+		gflog_debug(GFARM_MSG_1001451, "search_idle: no memory");
 		return (GFARM_ERR_NO_MEMORY);
+	}
 
 	i = 0;
 	for (h = search_idle_candidate_list; h != NULL; h = h->next)
@@ -1521,46 +1632,88 @@ select_hosts(struct gfm_connection *gfm_server,
 {
 	gfarm_error_t e;
 	int i;
+	gfarm_timerval_t t1, t2, t3, t4;
+
+	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t1);
+	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t2);
+	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t3);
+	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t4);
 
 	/*
 	 * if !gfm_client_is_connection_valid(gfm_server)
 	 *	gfm_client_username() shouldn't be called,
 	 */
-	if (!gfm_client_is_connection_valid(gfm_server))
+	if (!gfm_client_is_connection_valid(gfm_server)) {
+		gflog_debug(GFARM_MSG_1001452,
+		    "select_hosts: connection cache was purged");
 		return (GFARM_ERR_STALE_FILE_HANDLE);
+	}
 
+	gfs_profile(gfarm_gettimerval(&t1));
 	e = search_idle_candidate_list_init(gfm_server);
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001453,
+		    "gfarm_schedule_select_host: "
+		    "search_idle_candidate_list_init: %s",
+		    gfarm_error_string(e));
 		return (e);
+	}
+	gfs_profile(gfarm_gettimerval(&t2));
 	for (i = 0; i < ninfos; i++) {
 		e = search_idle_candidate_list_add(gfm_server, &infos[i]);
-		if (e != GFARM_ERR_NO_ERROR)
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001454,
+			    "gfarm_schedule_select_host: "
+			    "search_idle_candidate_list_add: %s",
+			    gfarm_error_string(e));
 			return (e);
+		}
 	}
-	return (acyclic ?
-	    search_idle(nohostsp, ohosts, oports, write_mode) :
-	    search_idle_cyclic(*nohostsp, ohosts, oports, write_mode));
+	gfs_profile(gfarm_gettimerval(&t3));
+	if (acyclic)
+		e = search_idle(nohostsp, ohosts, oports, write_mode);
+	else
+		e = search_idle_cyclic(*nohostsp, ohosts, oports, write_mode);
+	if (e != GFARM_ERR_NO_ERROR)
+		gflog_debug(GFARM_MSG_1001455,
+		    "gfarm_schedule_select_host: search result: %s",
+		    gfarm_error_string(e));
+
+	gfs_profile(gfarm_gettimerval(&t4));
+	gfs_profile(
+		gflog_info(GFARM_MSG_1002417,
+		    "(select_hosts) init %f, add %f, schedule %f",
+			   gfarm_timerval_sub(&t2, &t1),
+			   gfarm_timerval_sub(&t3, &t2),
+			   gfarm_timerval_sub(&t4, &t3)));
+
+	return (e);
 }
 
 gfarm_error_t
 gfarm_schedule_select_host(struct gfm_connection *gfm_server,
-	int nihosts, struct gfarm_host_sched_info *ihinfos,
+	int nhosts, struct gfarm_host_sched_info *infos,
 	int write_mode, char **hostp, int *portp)
 {
 	gfarm_error_t e;
-	int n, port, acyclic = 1;
 	char *host;
+	int port, n = 1;
 
-	n = 1;
-	e = select_hosts(gfm_server, acyclic, write_mode, nihosts, ihinfos,
+	e = select_hosts(gfm_server, 1, write_mode, nhosts, infos,
 	    &n, &host, &port);
 	if (e != GFARM_ERR_NO_ERROR)
 		return (e);
-	if (n == 0) /* although this shouldn't happen */
+	if (n == 0) { /* although this shouldn't happen */
+		gflog_debug(GFARM_MSG_1001456,
+		    "gfarm_schedule_select_host: no filesystem node");
 		return (GFARM_ERRMSG_NO_FILESYSTEM_NODE);
+	}
 	host = strdup(host);
-	if (host == NULL)
+	if (host == NULL) {
+		gflog_debug(GFARM_MSG_1001457,
+		    "gfarm_schedule_select_host: no memory");
 		return (GFARM_ERR_NO_MEMORY);
+	}
 
 	*hostp = host;
 	*portp = port;
@@ -2191,9 +2344,8 @@ statfsnode(char *canonical_hostname, int use_cache,
 		    &h->bsize,
 		    &h->blocks, &h->bfree, &h->bavail,
 		    &h->files, &h->ffree, &h->favail);
-		if (e != NULL || e2 != NULL) {
+		if (e != NULL || e2 != NULL)
 			return (e != NULL ? e : e2);
-		}
 		h->statfs_cache_time = search_idle_now;
 		h->flags |=
 		    HOST_STATE_FLAG_AUTH_SUCCEED|HOST_STATE_FLAG_STATFS_AVAIL;
