@@ -1,5 +1,5 @@
 /*
- * $Id: gfmd.c 4751 2010-06-16 13:53:28Z tatebe $
+ * $Id: gfmd.c 4712 2010-06-09 06:26:23Z n-soda $
  */
 
 #include <gfarm/gfarm_config.h>
@@ -32,8 +32,10 @@
 #include <gfarm/gfarm_misc.h>
 #include <gfarm/gfs.h>
 
-#include "liberror.h"
 #include "gfutil.h"
+#include "thrsubr.h"
+
+#include "liberror.h"
 #include "gfp_xdr.h"
 #include "io_fd.h"
 #include "hostspec.h"
@@ -46,14 +48,16 @@
 
 #include "../gfsl/gfarm_auth.h"
 
-#include "thrpool.h"
 #include "subr.h"
+#include "thrpool.h"
+#include "callout.h"
 #include "db_access.h"
 #include "host.h"
 #include "user.h"
 #include "group.h"
 #include "peer.h"
 #include "inode.h"
+#include "dead_file_copy.h"
 #include "process.h"
 #include "fs.h"
 #include "job.h"
@@ -80,10 +84,23 @@
 #define GFMD_CONNECTION_LIMIT	65536
 #endif
 
+#ifndef CALLOUT_NTHREADS
+/*
+ * this is number of thread pools which are used by callouts,
+ * so thrpool_add_job() in callout_main() won't be blocked for long time.
+ *
+ * currently, only back_channel_send_thread_pool is called from callouts.
+ */
+#define CALLOUT_NTHREADS	1
+#endif
+
 char *program_name = "gfmd";
 static struct protoent *tcp_proto;
 static char *pid_file;
 
+struct thread_pool *authentication_thread_pool;
+struct thread_pool *sync_protocol_thread_pool;
+
 /* this interface is exported for a use from a private extension */
 gfarm_error_t
 gfm_server_protocol_extension_default(struct peer *peer,
@@ -101,24 +118,23 @@ gfarm_error_t (*gfm_server_protocol_extension)(struct peer *,
 
 gfarm_error_t
 protocol_switch(struct peer *peer, int from_client, int skip, int level,
-	gfarm_int32_t *requestp, gfarm_error_t *on_errorp)
+	gfarm_int32_t *requestp, gfarm_error_t *on_errorp, int *suspendedp)
 {
 	gfarm_error_t e, e2;
-	int eof;
 	gfarm_int32_t request;
 
-	e = gfp_xdr_recv(peer_get_conn(peer), 0, &eof, "i", &request);
-	if (eof) {
-		/* actually, this is not an error, but completion on eof */
-		peer_record_protocol_error(peer);
-		return (GFARM_ERR_NO_ERROR);
-	}
+	e = gfp_xdr_recv_request_command(peer_get_conn(peer), 0, NULL,
+	    &request);
 	if (e != GFARM_ERR_NO_ERROR) {
-		gflog_warning(GFARM_MSG_1000180,
-		    "receiving request number: %s",
-		    gfarm_error_string(e));
-		peer_record_protocol_error(peer);
-		return (e); /* finish on error */
+		if (e == GFARM_ERR_UNEXPECTED_EOF) {
+			e = GFARM_ERR_NO_ERROR;
+		} else {
+			gflog_warning(GFARM_MSG_1000180,
+			    "receiving request number: %s",
+			    gfarm_error_string(e));
+		}
+		peer_record_protocol_error(peer); /* mark this peer finished */
+		return (e);
 	}
 	switch (request) {
 	case GFM_PROTO_HOST_INFO_GET_ALL:
@@ -295,14 +311,24 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 		e = gfm_server_getdirentsplus(peer, from_client, skip);
 		break;
 	case GFM_PROTO_REOPEN:
-		e = gfm_server_reopen(peer, from_client, skip);
+		e = gfm_server_reopen(peer, from_client, skip,
+		    suspendedp);
 		break;
 	case GFM_PROTO_CLOSE_READ:
 		e = gfm_server_close_read(peer, from_client, skip);
 		break;
+#ifdef COMPAT_GFARM_2_3
 	case GFM_PROTO_CLOSE_WRITE:
 		e = gfm_server_close_write(peer, from_client, skip);
 		break;
+#endif
+	case GFM_PROTO_CLOSE_WRITE_V2_4:
+		e = gfm_server_close_write_v2_4(peer, from_client, skip,
+		    suspendedp);
+		break;
+	case GFM_PROTO_GENERATION_UPDATED:
+		e = gfm_server_generation_updated(peer, from_client, skip);
+		break;
 	case GFM_PROTO_LOCK:
 		e = gfm_server_lock(peer, from_client, skip);
 		break;
@@ -315,11 +341,19 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 	case GFM_PROTO_LOCK_INFO:
 		e = gfm_server_lock_info(peer, from_client, skip);
 		break;
+#ifdef COMPAT_GFARM_2_3
 	case GFM_PROTO_SWITCH_BACK_CHANNEL:
 		e = gfm_server_switch_back_channel(peer, from_client, skip);
 		/* should not call gfp_xdr_flush() due to race */
 		*requestp = request;
 		return (e);
+#endif
+	case GFM_PROTO_SWITCH_ASYNC_BACK_CHANNEL:
+		e = gfm_server_switch_async_back_channel(peer,
+		    from_client, skip);
+		/* should not call gfp_xdr_flush() due to race */
+		*requestp = request;
+		return (e);
 	case GFM_PROTO_GLOB:
 		e = gfm_server_glob(peer, from_client, skip);
 		break;
@@ -359,8 +393,15 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 	case GFM_PROTO_REPLICA_REMOVE_BY_FILE:
 		e = gfm_server_replica_remove_by_file(peer, from_client, skip);
 		break;
+	case GFM_PROTO_REPLICA_INFO_GET:
+		e = gfm_server_replica_info_get(peer, from_client, skip);
+		break;
+	case GFM_PROTO_REPLICATE_FILE_FROM_TO:
+		e = gfm_server_replicate_file_from_to(peer, from_client, skip);
+		break;
 	case GFM_PROTO_REPLICA_ADDING:
-		e = gfm_server_replica_adding(peer, from_client, skip);
+		e = gfm_server_replica_adding(peer, from_client, skip,
+		    suspendedp);
 		break;
 	case GFM_PROTO_REPLICA_ADDED: /* obsolete protocol */
 		e = gfm_server_replica_added(peer, from_client, skip);
@@ -451,8 +492,10 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 		    from_client, skip, level, request, requestp, on_errorp);
 		break;
 	}
-	if ((level == 0 && request != GFM_PROTO_COMPOUND_BEGIN)
-	    || request == GFM_PROTO_COMPOUND_END) {
+
+	if (!*suspendedp &&
+	    ((level == 0 && request != GFM_PROTO_COMPOUND_BEGIN)
+	    || request == GFM_PROTO_COMPOUND_END)) {
 		/* flush only when a COMPOUND loop is done */
 		if (debug_mode)
 			gflog_debug(GFARM_MSG_1000182, "gfp_xdr_flush");
@@ -492,17 +535,20 @@ protocol_service(struct peer *peer)
 	gfarm_error_t e, dummy;
 	gfarm_int32_t request;
 	int from_client;
+	int suspended = 0;
 	int transaction = 0;
-	const char msg[] = "protocol_service";
+	static const char diag[] = "protocol_service";
 
 	from_client = peer_get_auth_id_type(peer) == GFARM_AUTH_ID_TYPE_USER;
 	if (ps->nesting_level == 0) { /* top level */
 		e = protocol_switch(peer, from_client, 0, 0,
-		    &request, &dummy);
+		    &request, &dummy, &suspended);
+		if (suspended)
+			return (1); /* finish */
 		giant_lock();
 		peer_fdpair_clear(peer);
 		if (peer_had_protocol_error(peer)) {
-			if (db_begin(msg) == GFARM_ERR_NO_ERROR)
+			if (db_begin(diag) == GFARM_ERR_NO_ERROR)
 				transaction = 1;
 			/*
 			 * the following internally calls inode_close*() and
@@ -512,18 +558,20 @@ protocol_service(struct peer *peer)
 			 */
 			peer_free(peer);
 			if (transaction)
-				db_end(msg);
+				db_end(diag);
 			giant_unlock();
 			return (1); /* finish */
 		}
 		giant_unlock();
 	} else { /* inside of a COMPOUND block */
 		e = protocol_switch(peer, from_client, cs->skip, 1,
-		    &request, &cs->current_part);
+		    &request, &cs->current_part, &suspended);
+		if (suspended)
+			return (1); /* finish */
 		if (peer_had_protocol_error(peer)) {
 			giant_lock();
 			peer_fdpair_clear(peer);
-			if (db_begin(msg) == GFARM_ERR_NO_ERROR)
+			if (db_begin(diag) == GFARM_ERR_NO_ERROR)
 				transaction = 1;
 			/*
 			 * the following internally calls inode_close*() and
@@ -533,7 +581,7 @@ protocol_service(struct peer *peer)
 			 */
 			peer_free(peer);
 			if (transaction)
-				db_end(msg);
+				db_end(diag);
 			giant_unlock();
 			return (1); /* finish */
 		}
@@ -542,6 +590,9 @@ protocol_service(struct peer *peer)
 			 * set cs->cause, if it's first error at a main part
 			 * of a COMPOUND block
 			 */
+			gflog_debug(GFARM_MSG_1001481,
+				"protocol_switch() failed inside of a "
+				"COMPOUND block: %s", gfarm_error_string(e));
 			if (cs->cause == GFARM_ERR_NO_ERROR && !cs->skip)
 				cs->cause = e;
 			cs->skip = 1;
@@ -554,10 +605,17 @@ protocol_service(struct peer *peer)
 			cs->skip = cs->current_part != cs->cause;
 		}
 	}
-	if (request == GFM_PROTO_SWITCH_BACK_CHANNEL) {
+	if (
+#ifdef COMPAT_GFARM_2_3
+	    request == GFM_PROTO_SWITCH_BACK_CHANNEL ||
+#endif
+	    request == GFM_PROTO_SWITCH_ASYNC_BACK_CHANNEL) {
 		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001482,
+				"failed to process GFM_PROTO_SWITCH_BACK_"
+				"CHANNEL request: %s", gfarm_error_string(e));
 			giant_lock();
-			if (db_begin(msg) == GFARM_ERR_NO_ERROR)
+			if (db_begin(diag) == GFARM_ERR_NO_ERROR)
 				transaction = 1;
 			/*
 			 * the following internally calls inode_close*() and
@@ -567,7 +625,7 @@ protocol_service(struct peer *peer)
 			 */
 			peer_free(peer);
 			if (transaction)
-				db_end(msg);
+				db_end(diag);
 			giant_unlock();
 		}
 		return (1); /* finish */
@@ -586,6 +644,12 @@ protocol_main(void *arg)
 {
 	struct peer *peer = arg;
 
+	/*
+	 * the reason we call peer_invoked() here is just for consistency,
+	 * because currently this is unnecessary for a foreground channel.
+	 */
+	peer_invoked(peer);
+
 	do {
 		if (protocol_service(peer))
 			return (NULL); /* end of gfmd protocol session */
@@ -614,6 +678,81 @@ protocol_main(void *arg)
 	return (NULL);
 }
 
+struct resuming_queue {
+	pthread_mutex_t mutex;
+	pthread_cond_t nonempty;
+	struct event_waiter *queue;
+} resuming_pendings = {
+	PTHREAD_MUTEX_INITIALIZER,
+	PTHREAD_COND_INITIALIZER,
+	NULL
+};
+
+void
+resuming_enqueue(struct event_waiter *entry)
+{
+	struct resuming_queue *q = &resuming_pendings;
+	static const char diag[] = "resuming_enqueue";
+
+	gfarm_mutex_lock(&q->mutex, diag, "mutex");
+	entry->next = q->queue;
+	q->queue = entry;
+	gfarm_cond_signal(&q->nonempty, diag, "nonempty");
+	gfarm_mutex_unlock(&q->mutex, diag, "mutex");
+}
+
+struct event_waiter *
+resuming_dequeue(struct resuming_queue *q, const char *diag)
+{
+	struct event_waiter *entry;
+
+	gfarm_mutex_lock(&q->mutex, diag, "mutex");
+	while  (q->queue == NULL)
+		gfarm_cond_wait(&q->nonempty, &q->mutex, diag, "nonempty");
+	entry = q->queue;
+	q->queue = entry->next;
+	gfarm_mutex_unlock(&q->mutex, diag, "mutex");
+	return (entry);
+}
+
+void *
+resuming_thread(void *arg)
+{
+	struct event_waiter *entry = arg;
+	struct peer *peer = entry->peer;
+	int suspended = 0;
+
+	(*entry->action)(peer, entry->arg, &suspended);
+	free(entry);
+	if (suspended)
+		return (NULL);
+
+	if (gfp_xdr_recv_is_ready(peer_get_conn(peer)))
+		protocol_main(peer);
+
+	/* this return value won't be used, because this thread is detached */
+	return (NULL);
+}
+
+
+void *
+resumer(void *arg)
+{
+	struct event_waiter *entry;
+	static const char diag[] = "resumer";
+
+	for (;;) {
+		entry = resuming_dequeue(&resuming_pendings, diag);
+
+		thrpool_add_job(sync_protocol_thread_pool,
+		    resuming_thread, entry);
+
+	}
+
+	/*NOTREACHED*/
+	return (NULL);
+}
+
 /* only called in case of gfarm_auth_id_type == GFARM_AUTH_ID_TYPE_USER */
 gfarm_error_t
 auth_uid_to_global_username(void *closure,
@@ -623,6 +762,7 @@ auth_uid_to_global_username(void *closure,
 {
 	char *global_username;
 	struct user *u;
+	static const char diag[] = "auth_uid_to_global_username";
 
 	giant_lock();
 	if (GFARM_IS_AUTH_GSI(auth_method)) { /* auth_user_id is a DN */
@@ -637,11 +777,13 @@ auth_uid_to_global_username(void *closure,
 		 * do not return GFARM_ERR_NO_SUCH_USER
 		 * to prevent information leak
 		 */
+		gflog_debug(GFARM_MSG_1001483,
+			"lookup for user failed");
 		return (GFARM_ERR_AUTHENTICATION);
 	}
 	if (global_usernamep == NULL)
 		return (GFARM_ERR_NO_ERROR);
-	global_username = strdup(user_name(u));
+	global_username = strdup_log(user_name(u), diag);
 	if (global_username == NULL)
 		return (GFARM_ERR_NO_MEMORY);
 	*global_usernamep = global_username;
@@ -659,6 +801,7 @@ peer_authorize(struct peer *peer)
 	struct sockaddr addr;
 	socklen_t addrlen = sizeof(addr);
 	char addr_string[GFARM_SOCKADDR_STRLEN];
+	static const char diag[] = "peer_authorize";
 
 	/* without TCP_NODELAY, gfmd is too slow at least on NetBSD-3.0 */
 	rv = 1;
@@ -677,13 +820,11 @@ peer_authorize(struct peer *peer)
 		gfarm_sockaddr_to_string(&addr,
 		    addr_string, GFARM_SOCKADDR_STRLEN);
 		gflog_warning(GFARM_MSG_1000185,
-		    "%s: %s", gfarm_error_string(e), addr_string);
-		hostname = strdup(addr_string);
-		if (hostname == NULL) {
-			gflog_warning(GFARM_MSG_1000186, "%s: %s", addr_string,
-			    gfarm_error_string(GFARM_ERR_NO_MEMORY));
+		    "gfarm_sockaddr_to_name(%s): %s",
+		    gfarm_error_string(e), addr_string);
+		hostname = strdup_log(addr_string, diag);
+		if (hostname == NULL)
 			return (GFARM_ERR_NO_MEMORY);
-		}
 	}
 	e = gfarm_authorize(peer_get_conn(peer), 0, GFM_SERVICE_TAG,
 	    hostname, &addr, auth_uid_to_global_username, NULL,
@@ -744,7 +885,8 @@ accepting_loop(int accepting_socket)
 			    "peer_alloc: %s", gfarm_error_string(e));
 			close(client_socket);
 		} else {
-			thrpool_add_job(try_auth, peer);
+			thrpool_add_job(authentication_thread_pool,
+			    try_auth, peer);
 		}
 	}
 }
@@ -792,24 +934,33 @@ write_pid()
 	if (pid_fp == NULL)
 		gflog_fatal_errno(GFARM_MSG_1000196, "open: %s", pid_file);
 
-	fprintf(pid_fp, "%ld\n", (long)getpid());
-	fclose(pid_fp);
+	if (fprintf(pid_fp, "%ld\n", (long)getpid()) == -1)
+		gflog_error_errno(GFARM_MSG_1002351,
+		    "writing PID to %s", pid_file);
+	if (fclose(pid_fp) != 0)
+		gflog_error_errno(GFARM_MSG_1002352, "fclose(%s)", pid_file);
 }
 
 void
 sigs_set(sigset_t *sigs)
 {
-	sigemptyset(sigs);
+	if (sigemptyset(sigs) == -1)
+		gflog_fatal_errno(GFARM_MSG_1002353, "sigemptyset()");
 	sigaddset(sigs, SIGHUP);
 #ifdef __NetBSD__ /* NetBSD 4 delivers SIGINT to gfmd even under gdb */
 	if (!debug_mode)
 #endif
-		sigaddset(sigs, SIGINT);
-	sigaddset(sigs, SIGTERM);
+		if (sigaddset(sigs, SIGINT) == -1)
+			gflog_fatal_errno(GFARM_MSG_1002354,
+			    "sigaddset(SIGINT)");
+	if (sigaddset(sigs, SIGTERM) == -1)
+		gflog_fatal_errno(GFARM_MSG_1002355, "sigaddset(SIGTERM)");
 #ifdef SIGINFO
-	sigaddset(sigs, SIGINFO);
+	if (sigaddset(sigs, SIGINFO) == -1)
+		gflog_fatal_errno(GFARM_MSG_1002356, "sigaddset(SIGINFO)");
 #endif
-	sigaddset(sigs, SIGUSR2);
+	if (sigaddset(sigs, SIGUSR2) == -1)
+		gflog_fatal_errno(GFARM_MSG_1002357, "sigaddset(SIGUSR2)");
 }
 
 void *
@@ -818,7 +969,7 @@ sigs_handler(void *p)
 	sigset_t *sigs = p;
 	int sig;
 	int transaction = 0;
-	const char msg[] = "sigs_handler";
+	static const char diag[] = "sigs_handler";
 
 #ifdef __linux__
 	/* A Linux Thread is a process having its own process id. */
@@ -876,11 +1027,8 @@ sigs_handler(void *p)
 	/* so, it's safe to modify the state of all peers */
 	giant_lock();
 
-	gflog_info(GFARM_MSG_1000200, "dumping dead file copies");
-	host_remove_replica_dump_all();
-
 	gflog_info(GFARM_MSG_1000201, "shutting down peers");
-	if (db_begin(msg) == GFARM_ERR_NO_ERROR)
+	if (db_begin(diag) == GFARM_ERR_NO_ERROR)
 		transaction = 1;
 	/*
 	 * the following internally calls inode_close*() and
@@ -889,7 +1037,7 @@ sigs_handler(void *p)
 	 */
 	peer_shutdown_all();
 	if (transaction)
-		db_end(msg);
+		db_end(diag);
 
 	/* save all pending transactions */
 	/* db_terminate() needs giant_lock(), see comment in dbq_enter() */
@@ -921,18 +1069,38 @@ usage(void)
 void
 gfmd_modules_init_default(int table_size)
 {
+	authentication_thread_pool = thrpool_new(gfarm_metadb_thread_pool_size,
+	    gfarm_metadb_job_queue_length, "authentication threads");
+	sync_protocol_thread_pool = thrpool_new(gfarm_metadb_thread_pool_size,
+	    gfarm_metadb_job_queue_length, "synchronous protocol threads");
+	if (authentication_thread_pool == NULL ||
+	    sync_protocol_thread_pool == NULL)
+		gflog_fatal(GFARM_MSG_1001485,
+		    "thread pool size:%d, queue length:%d: no memory",
+		    gfarm_metadb_thread_pool_size,
+		    gfarm_metadb_job_queue_length);
+
+	callout_module_init(CALLOUT_NTHREADS);
+
+	back_channel_init();
+
+	/* directory service */
 	host_init();
 	user_init();
 	group_init();
+
+	/* filesystem */
 	inode_init();
 	dir_entry_init();
 	file_copy_init();
-	dead_file_copy_init();
 	symlink_init();
 	xattr_init();
 	quota_init();
 
-	peer_init(table_size, protocol_main);
+	/* must be after hosts and filesystem */
+	dead_file_copy_init();
+
+	peer_init(table_size, sync_protocol_thread_pool, protocol_main);
 	job_table_init(table_size);
 }
 
@@ -942,6 +1110,8 @@ void (*gfmd_modules_init)(int); /* intentionally remains uninitialized */
 int
 main(int argc, char **argv)
 {
+	extern char *optarg;
+	extern int optind;
 	gfarm_error_t e;
 	char *config_file = NULL, *port_number = NULL;
 	int syslog_level = -1;
@@ -1006,6 +1176,9 @@ main(int argc, char **argv)
 		gfarm_config_set_filename(GFMD_CONFIG);
 	e = gfarm_server_initialize();
 	if (e != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001486,
+			"gfarm_server_initialize() failed: %s",
+		    gfarm_error_string(e));
 		fprintf(stderr, "gfarm_server_initialize: %s\n",
 		    gfarm_error_string(e));
 		exit(1);
@@ -1022,7 +1195,6 @@ main(int argc, char **argv)
 	 */
 	write_pid(pid_file);
 
-	thrpool_init();
 	giant_init();
 
 	table_size = GFMD_CONNECTION_LIMIT;
@@ -1068,7 +1240,7 @@ main(int argc, char **argv)
 	if (!debug_mode) {
 		gflog_syslog_open(LOG_PID, syslog_facility);
 		if (gfarm_daemon(0, 0) == -1)
-			gflog_warning_errno(GFARM_MSG_UNFIXED, "daemon");
+			gflog_warning_errno(GFARM_MSG_1001487, "daemon");
 	}
 	/*
 	 * We do this after calling gfarm_daemon(),
@@ -1091,16 +1263,19 @@ main(int argc, char **argv)
 	if (e != GFARM_ERR_NO_ERROR)
 		gflog_fatal(GFARM_MSG_1000210,
 		    "create_detached_thread(sigs_handler): %s",
-			    gfarm_error_string(e));
+		    gfarm_error_string(e));
 
 	e = create_detached_thread(db_thread, NULL);
 	if (e != GFARM_ERR_NO_ERROR)
 		gflog_fatal(GFARM_MSG_1000211,
 		    "create_detached_thread(db_thread): %s",
-			    gfarm_error_string(e));
+		    gfarm_error_string(e));
 
-	/* check and repair nlink */
-	inode_nlink_check();
+	e = create_detached_thread(resumer, NULL);
+	if (e != GFARM_ERR_NO_ERROR)
+		gflog_fatal(GFARM_MSG_1002209,
+		    "create_detached_thread(resumer): %s",
+		    gfarm_error_string(e));
 
 	accepting_loop(sock);
 
