@@ -93,17 +93,28 @@ dir_foreach(
 		else
 			return (GFARM_ERR_NO_ERROR);
 	}
-	if (!S_ISDIR(st.st_mode))
+	if (!S_ISDIR(st.st_mode)) {
+		gflog_debug(GFARM_MSG_1002204,
+			"Invalid argument st.st_mode");
 		return (GFARM_ERR_INVALID_ARGUMENT); /* XXX */
+	}
 
 	if (op_dir1 != NULL) {
 		e = op_dir1(dir, &st, arg);
-		if (e != GFARM_ERR_NO_ERROR)
+		if (e != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1002205,
+				"op_dir1() failed: %s",
+				gfarm_error_string(e));
 			return (e);
+		}
 	}
 	dirp = opendir(dir);
-	if (dirp == NULL)
-		return (gfarm_errno_to_error(errno));
+	if (dirp == NULL) {
+		int err = errno;
+		gflog_debug(GFARM_MSG_1002206, "opendir() failed: %s",
+		    strerror(err));
+		return (gfarm_errno_to_error(err));
+	}
 
 	/* if dir is '.', remove it */
 	if (dir[0] == '.' && dir[1] == '\0')
@@ -116,6 +127,8 @@ dir_foreach(
 		GFARM_MALLOC_ARRAY(dir1, strlen(dir) + strlen(dp->d_name) + 2);
 		if (dir1 == NULL) {
 			closedir(dirp);
+			gflog_debug(GFARM_MSG_1002207,
+				"allocation of array 'dir1' failed");
 			return (GFARM_ERR_NO_MEMORY);
 		}
 		strcpy(dir1, dir);
@@ -181,8 +194,6 @@ delete_invalid_file_or_directory(char *pathname)
 	return (e);
 }
 
-extern const char READONLY_CONFIG_FILE[];
-
 static gfarm_error_t
 fixfrag(char *path)
 {
@@ -192,10 +203,6 @@ fixfrag(char *path)
 	gfarm_error_t e;
 	struct stat st;
 
-	/* READONLY_CONFIG_FILE should be skipped */
-	if (strcmp(path, READONLY_CONFIG_FILE) == 0)
-		return (GFARM_ERR_NO_ERROR);
-
 	if (get_inum_gen(path, &inum, &gen))
 		return (delete_invalid_file_or_directory(path));
 	if (stat(path, &st))
